#ifndef _FILEIO_H
#define _FILEIO_H

#include<string>
#include<fstream>
using namespace std;

enum fileIO_StatusType {READY,NONE,OPEN_FAIL};
//READY = The file is successfully opened and ready to access file
//NONE = not opened yet
//OPEN_FAIL = Failed to open file
//**Will be added more error code later  e.g.)READ_FAIL, WRITE_FAIL

///////////////////////////////////////////////////////////////////
//CLASS fileIO
//Function : handle file Input/Output
///////////////////////////////////////////////////////////////////
//PRIVATE MEMBER VARIABLES :
//TYPE               NAME         USAGE
//-----------------------------------------------
//string             FileName     Keep filename
//fstream            file         File i/o stream
//fileIO_StatusType  status       Keep status of i/o stream
///////////////////////////////////////////////////////////////////
//PUBLIC MEMBER FUNCTIONS :
//Constructors : 
//	fileIO()             - DO NOT open file
//	fileIO(string FName) - Open file FName
//                         If FName file does not exist, open fails
//  fileIO(string FNAME,
//         bool Makefile - Open file FName
//                         If Makefile is true, open create FName file
//                         when FName file doesn't exist.(**NOT IMPLEMENTED YET)
//------------------------------------------------------------------------------
//~fileIO()   - If a file is opened, close file.
//open()      - Open file FileName
//close()     - Close file FileName
//read2buffer - Read size bytes of data from file to buffer at beg_pointer
//writebuffer - Write size bytes of data from buffer to file at beg_pointer
//getStatus   - Return current status of file
//getFileName - Return FileName
////////////////////////////////////////////////////////////////////
//IN NEAR FUTURE :
//1. MORE ERROR CHECKING
//2. ENCRYPTION & DECRYTION of DATA
//3. CREATE FILE WHEN NOT EXIST
////////////////////////////////////////////////////////////////////

class fileIO
{

private :
	string FileName;
	fstream file; 
	fileIO_StatusType status;

public:
	fileIO();
	fileIO(string FName);
	fileIO(string FName, bool Makefile);
	
	~fileIO();
	
	void open(string FName, bool Makefile);
	void close();
	void read2buffer(char* buffer, int beg_pointer, int size);
	//read size bytes from file to buffer at beg_pointer
	void writebuffer(char* buffer, int beg_pointer, int size);
	//write size bytes to file at  start_pointer
	void writeAttheEnd(char* buffer, int size);
	fileIO_StatusType getStatus();
	//return current status
	string getFileName();

};

#endif