
#ifndef JAGUAREX_H
#define JAGUAREX_H

#include "JaguarX.h"


class Filter
{
public:
	class Action
	{
	public:
	       enum Value
		   {
			   Invalid = -1, SetType, ForwardTo, ReplyWith, StoreOffline, Continue
		   };
		   Action(Value v = SetType, const string& param = "")
			   : _value(v), _param(param) {}
		   // Accessors
		   bool requires_param()
		   { return Action::ParamReq[_value]; }
		   const string& param() const 
		   { return _param; }
		   Value value() const 
		   { return _value; }
		   string toXML() const;
		   // Mutators
		   Action& operator<<(Value v) 
		   { _value = v; return *this;}
		   Action& operator<<(const string& param) {
			   _param = param; return *this; }
		   // Translator
		   static Value translate(const string& s);
	private:
	       static const bool ParamReq[5];
		   Value  _value;
		   string _param;
	};
	
	class Condition
	{
	public:
	       enum Value
		   {
			   Invalid = -1, Unavailable, From, MyResourceEquals, SubjectEquals, BodyEquals, ShowEquals, TypeEquals
		   };
		   Condition(Value v = Unavailable, const string& param = "")
			   : _value(v), _param(param) {}
		   // Accessors
		   bool  requires_param() 
		   { return Condition::ParamReq[_value]; }
		   const string& param() const 
		   { return _param; }
		   Value value() const 
		   { return _value; }
		   string toXML() const;
		   // Mutators
		   Condition& operator<<(Value v) 
		   { _value = v; return *this;}
		   Condition& operator<<(const string& param) 
		   { _param = param; return *this; }
		   // Translator
		   static Value translate(const string& s);
	private:
	       static const bool ParamReq[7];
		   Value  _value;
		   string _param;
	};
	
	typedef list<Action> ActionList;
	typedef list<Condition> ConditionList;
	
	public:
		// Constructors
		Filter(const string& name);
		Filter(const Element& rule);
		Filter(const Filter& f);
		bool operator==(const Filter& f) const { return &f == this; }
		bool operator==(const Filter& f) { return &f == this; }
		// XML converter
		string toXML() const;
		// Accessors
		ActionList&    Actions()    { return _actions; }
		ConditionList& Conditions() { return _conditions; }
		const string&  Name() const { return _name; }
		void           setName(const string& newname) { _name = newname; }
	private:
		ActionList    _actions;
		ConditionList _conditions;
		string _name;
};

class FilterList
: public list<Filter>
{
public:
	FilterList(const Element& query);
	string toXML() const;
};

//Agent Ŭ Ʈ ʿ䰡 ֳ?
class Agent : public JSig
{
public:
	Agent(const Agent& a);
	Agent(const string& jid, const Element& baseElement, JSession& s);
	~Agent();
public:
	// Events
//	Signal1<void, bool, Marshal<void> > evtFetchComplete;
	void(*evtFetchComplete)(bool);
	// Accessors
	const string& JID()         const { return _jid; };
	const string& name()        const { return _name; };
	const string& description() const { return _desc; };
	const string& service()     const { return _service; };
	const string& transport()   const { return _transport; };
	// Info ops
	bool isRegisterable() const { return _registerable; };
	bool isSearchable()   const { return _searchable; };
	bool isGCCapable()    const { return _gc_capable; };
	bool hasAgents()      const { return _subagents; };
	// Action ops
//	void requestRegister(ElementCallbackFunc f);
//	void requestSearch(ElementCallbackFunc f);
//	void requestRegister(void(*func)(const Element& e));
//	void requestSearch(void(*func)(const Element& e));
	void requestRegister(SigFuncElement &s);
	void requestSearch(SigFuncElement &s);
	void fetch();

	friend void Callback_on_fetch(JSig * inst, const Element& iq);
protected:
	void on_fetch(const Element& iq);
private:
	SigFuncElement _Sigon_fetch;
	string   _jid;
	string   _name;
	string   _desc;
	string   _service;
	string   _transport;
	bool     _registerable;
	bool     _searchable;
	bool     _subagents;
	bool     _gc_capable;
	bool     _fetchrequested;
	JSession& _session;
	
};

class AgentList
: public list<Agent>
{
public:
	AgentList(const Element& query, JSession& s);
	void load(const Element& query, JSession& s);
};




#endif
