


#include "JaguarX.h"

using namespace JaguarX;


const unsigned int JMessage::numTypes = 5;

JMessage::JMessage(const Element& t)
     : JPacket(t)
{
     // Determine message type..
     _type = translateType(t.getAttrib("type"));

#ifndef WIN32
     Element *x;
     string date;

     x = findX("jabber:x:delay");
     if (x) 
     {
	  date = x->getAttrib("stamp");
	  if (!date.empty()) 
	  {
	       struct tm ts;
	       memset(&ts, 0, sizeof(struct tm));
	       sscanf(date.c_str(), "%04d%02d%02dT%02d:%02d:%02d", &ts.tm_year, &ts.tm_mon, &ts.tm_mday, 
		      &ts.tm_hour, &ts.tm_min, &ts.tm_sec);
	       ts.tm_year -= 1900;
	       ts.tm_mon -= 1;
	       // ok, lets make this time local
#ifdef HAVE_TIMEGM
	       tzset(); // this should be called in the beginning anyways..
	       ts.tm_hour += timezone;
	       _timestamp = timegm(&ts); 
#else // HAVE_TIMEGM
	       char *tz = getenv("TZ");
	       setenv("TZ", "", 1);
	       tzset();
	       _timestamp = mktime(&ts);
	       if (tz)
		     setenv("TZ", tz, 1);
	       else
		     unsetenv("TZ");
	       tzset();
#endif // HAVE_TIMEGM
	       return;
	  }
     }
     // Otherwise we want to get the current time
     _timestamp = time(0);
     // And now we'll add the XML for it in case the XML of this JMessage
     // is logged or passed on
     x = addX("jabber:x:delay");
     x->putAttrib("from", getFrom());
     x->putAttrib("stamp", getDateTime("%Y%m%dT%H:%M:%S%z"));
#endif     
}

JMessage::JMessage(const string& jid, const string& body, JMessage::Type mtype)
     : JPacket("message")
{
     setTo(jid);
     if (!body.empty())
	  _base.addElement("body", body);
//     _timestamp = time(0);
     setType(mtype);
}
/*
JMessage::JMessage(const, string& jid, const unsigned short& body, JMessage::Type mtype)
	: JPacket("message")
{
     setTo(jid);
//     if (!strlen(body))
	  _base.addElement("body", body);
//     _timestamp = time(0);
     setType(mtype);
}
*/
void JMessage::setBody(const string& body)
{
     Element* body_tag = _base.findElement("body");
     if (body_tag)
     {
	  Element::iterator it = body_tag->begin();
	  for (; it != body_tag->end(); ++it )
	  {
	       if ((*it)->getType() != Node::ntCDATA)
		    continue;
	       (static_cast<CDATA*>(*it))->setText(body.c_str(), body.length());
          }
     }
     else
	  _base.addElement("body", body);
}

void JMessage::setSubject(const string& subject)
{
     Element* subject_tag = _base.findElement("subject");
     if (subject_tag)
     {
	  Element::iterator it = subject_tag->begin();
	  for (; it != subject_tag->end(); ++it )
	  {
	       if ((*it)->getType() != Node::ntCDATA)
		    continue;
	       (static_cast<CDATA*>(*it))->setText(subject.c_str(), subject.length());
	  }
     }
     else
	  _base.addElement("subject", subject);
}

void JMessage::setThread(const string& thread)
{
     Element* thread_tag = _base.findElement("thread");
     if (thread_tag)
     {
	  Element::iterator it = thread_tag->begin();
	  for (; it != thread_tag->end(); ++it )
	  {
	       if ((*it)->getType() != Node::ntCDATA)
		    continue;
	       (static_cast<CDATA*>(*it))->setText(thread.c_str(), thread.length());
          }
     }
     else
	  _base.addElement("thread", thread);
}

void JMessage::setType(JMessage::Type mtype)
{
     _base.putAttrib("type", translateType(mtype));
     _type      = mtype;
}

void JMessage::requestDelivered()
{
     Element* x = findX("jabber:x:event");
     if (x != NULL)
     {
	  x->addElement("delivered");
     }
     else
     {
	  Element* new_x = addX("jabber:x:event");
	  new_x->addElement("delivered");
     }
}

void JMessage::requestDisplayed()
{
     Element* x = findX("jabber:x:event");
     if (x != NULL)
     {
	  x->addElement("displayed");
     }
     else
     {
	  Element* new_x = addX("jabber:x:event");
	  new_x->addElement("displayed");
     }
}

void JMessage::requestComposing()
{
     Element* x = findX("jabber:x:event");
     if (x != NULL)
     {
	  x->addElement("composing");
     }
     else
     {
	  Element* new_x = addX("jabber:x:event");
	  new_x->addElement("composing");
     }
}

const string JMessage::getBody() const
{
     return string(_base.getChildCData("body"));
}

const string JMessage::getSubject() const
{
     return string(_base.getChildCData("subject"));
}

const string JMessage::getThread() const
{
     return string(_base.getChildCData("thread"));
}

JMessage::Type JMessage::getType() const
{
     return _type;
}

// added format for the future, for custom date formats in the logs
const string JMessage::getDateTime(const string& format) const
{
#ifndef WIN32
     char timestr[1024];
     struct tm *timestamp;

     timestamp = localtime(&_timestamp);
     if (format.empty())
	  strftime(timestr, 1024, _dtFormat.c_str(), timestamp);
     else
	  strftime(timestr, 1024, format.c_str(), timestamp);
     return string(timestr);
#else
    return "N/A";
#endif   
}

JMessage::JMessage(const JMessage& m, const string& body)
     : JPacket("Message")
{
     // Setup basic stuff
     setTo(m.getFrom());
     setType(m.getType());
     _base.addElement("body", body);

     // Get thread if available..
     string thread = m.getThread();
     if (!thread.empty())
	  _base.addElement("thread", thread);
}

JMessage JMessage::replyTo(const string& body) const
{
     return JMessage(*this, body);
}

JMessage JMessage::delivered() const
{
     JMessage new_m(getFrom(), "", getType()); // Create an empty JMessage
     string thread = getThread();
     if (!thread.empty())
	  new_m.setThread(thread);
     Element* new_x = new_m.addX("jabber:x:event");
     new_x->addElement("delivered");
     if (!getID().empty())
	  new_x->addElement("id", getID());

     return new_m;
}

JMessage JMessage::displayed() const
{
     JMessage new_m(getFrom(), "", getType()); // Create an empty JMessage
     string thread = getThread();
     if (!thread.empty())
	  new_m.setThread(thread);
     Element* new_x = new_m.addX("jabber:x:event");
     new_x->addElement("displayed");
     if (!getID().empty())
	  new_x->addElement("id", getID());

     return new_m;
}

JMessage JMessage::composing() const
{
     JMessage new_m(getFrom(), "", getType()); // Create an empty JMessage
     string thread = getThread();
     if (!thread.empty())
	  new_m.setThread(thread);
     Element* new_x = new_m.addX("jabber:x:event");
     new_x->addElement("composing");
     if (!getID().empty())
	  new_x->addElement("id", getID());

     return new_m;
}


string JMessage::translateType(Type mtype)
{
     switch(mtype)
     {
     case mtNormal:
	  return "normal";
     case mtError:
	  return "error";
     case mtChat:
	  return "chat";
     case mtGroupchat:
	  return "groupchat";
     case mtHeadline:
	  return "headline";
     }
     return "normal";
}

JMessage::Type JMessage::translateType(const string& mtype)
{
     // Determine what type of JMessage packet this is
     if (mtype == "")
	  return mtNormal;
     else if (mtype == "normal")
	  return mtNormal;
     else if (mtype == "error")
	  return mtError;
     else if (mtype == "chat")
	  return mtChat;
     else if (mtype == "groupchat")
	  return mtGroupchat;
     else if (mtype == "headline")
	  return mtHeadline;
     return mtNormal;
}

// This, unfortunately, is locale-independent
//string JMessage::_dtFormat  = "%d %b %Y %H:%M:%S";
string JMessage::_dtFormat  = "%c";

void JMessage::setDateTimeFormat(const string& format)
{
     _dtFormat = format;
}

const string& JMessage::getDateTimeFormat()
{
     return _dtFormat;
}
