
#ifndef MESSAGE_H
#define MESSAGE_H

#include "JaguarX.h"
#include "Packet.h"


namespace JaguarX
{


class JMessage
: public JPacket
{
public:
	static const unsigned int numTypes;
	enum Type {
		mtNormal,    /**< A normal JMessage (email, icq). */
			mtError,     /**< An error JMessage. */
			mtChat,      /**< A chat JMessage (AIM, IRC /msg's). */
			mtGroupchat, /**< A groupchat JMessage (IRC). */
			mtHeadline   /**< A headline JMessage (stock ticker, news ticker). */
	};
	
	JMessage(const Element& t);
	JMessage(const string& jid, const string& body, Type mtype = mtNormal);
//	JMessage(const string& jid, const unsigned short& body, Type mtype = mtNormal);
	void  setBody(const string& body);
	void  setSubject(const string& subject);
	void  setThread(const string& thread);
	void  setType(JMessage::Type mtype);
	void requestDelivered();
	void requestDisplayed();
	void requestComposing();
	const string getBody()     const;
	const string getSubject()  const;
	const string getThread()   const;
	Type         getType()     const;
	const string getDateTime(const string& format = "") const;
	JMessage replyTo(const string& body) const;
	JMessage JMessage::delivered() const;
	JMessage JMessage::displayed() const;
	JMessage JMessage::composing() const;
	static void setDateTimeFormat(const string& format);
	static const string& getDateTimeFormat();
protected:
		  // Specialized reply-to constructor
		  JMessage(const JMessage& m, const string& body);
		  
		  static string translateType(Type mtype);
		  static Type   translateType(const string& mtype);
private:
		  Type  _type;
		  time_t _timestamp;
		  static string _dtFormat;
};


}//end of JaguarX namespace

#endif