
#pragma warning(disable:4786)


#include "JaguarX.h"

using namespace JaguarX;

JPresence::JPresence(const Element& t)
     : JPacket(t)
{
	//XML ͸ ʱȭؼ JPresence ؾ Ѵ.
	// <presence type ='7 type'> <show> some data </show> </presence>
	// Determine the JPresence type
     _type = translateType(t.getAttrib("type"));
     // Determine the show value
     _show = translateShow(_type, t.getChildCData("show"));
}

JPresence::JPresence(const string& jid, Type ptype, Show stype , 
					 const string& status , const string& priority )
/*
JPresence::JPresence(const string& jid, JPresence::Type ptype, 
					 JPresence::Show stype, const string& status, 
					 const string& priority)*/
     : JPacket("presence")
{
     if (!jid.empty())
	  setTo(jid);
     setType(ptype);
     setStatus(status);
     setShow(stype);
     setPriority(priority);
}

void JPresence::setType(JPresence::Type ptype)
{
     if (ptype != JPresence::ptAvailable)
	  _base.putAttrib("type", translateType(ptype));
     _type = ptype;
}

void JPresence::setStatus(const string& status)
{
     if (!status.empty())
	  _base.addElement("status", status);
}

void JPresence::setShow(JPresence::Show stype)
{
     if (stype > stOnline)
     {
	  _base.addElement("show", translateShow(stype));
     }
     _show = stype;
}

void JPresence::setPriority(const string& priority)
{
     if (priority != "0")
     {
	  _base.addElement("priority", priority);
     }
     _priority = atoi(priority.c_str());
}

JPresence::Type JPresence::getType() const
{
     return _type;
}

const string JPresence::getStatus() const
{
     if (_base.getChildCData("status").empty())
	  return _base.getChildCData("error");
     else
	  return _base.getChildCData("status");
}

JPresence::Show JPresence::getShow() const
{
     return _show;
}

const string JPresence::getShow_str() const
{
     return translateShow(_show);
}

int JPresence::getPriority() const
{
     if (_type == JPresence::ptUnavailable)
	  return -1;
     else
	  return atoi(_base.getChildCData("priority").c_str());
}

string JPresence::translateType(Type ptype)
{
     switch(ptype)
     {
     case ptAvailable: 
	  return string("");
     case ptUnavailable: 
	  return string("unavailable");
     case ptSubRequest: 
	  return string("subscribe");
     case ptUnsubRequest: 
	  return string("unsubscribe");
     case ptSubscribed : 
	  return string("subscribed");
     case ptUnsubscribed:
	  return string("unsubscribed");
     case ptError:
	  return string("error");
     case ptInvisible:
	  return string("invisible");
     }
     return string("");
}

string JPresence::translateShow(Show stype)
{
     switch(stype)
     {
     case stInvalid:
	  return string("");
     case stOnline:
	  return string("online");
     case stOffline:
	  return string("offline");
     case stChat:
	  return string("chat");
     case stAway:
	  return string("away");
     case stXA:
	  return string("xa");
     case stDND:
	  return string("dnd");
     }
     return string("");
}

JPresence::Type JPresence::translateType(const string& ptype)
{
     // Determine what type of JPresence JPacket this is
     if (ptype.empty())
	  return ptAvailable;
     else if (ptype == "subscribe")
	  return ptSubRequest;
     else if (ptype == "unsubscribe")
	  return ptUnsubRequest;
     else if (ptype == "subscribed")
	  return ptSubscribed;
     else if (ptype == "unsubscribed")
	  return ptUnsubscribed;
     else if (ptype == "error")
	  return ptError;
     else if (ptype == "invisible")
	  return ptInvisible;
     else
	  return ptUnavailable;
}

JPresence::Show JPresence::translateShow(Type ptype, const string& show)
{
     // Determine what value show should return
     if (ptype == ptError)
	  return stOffline;
     else if ((ptype == ptAvailable) && (show.empty()))
	  return stOnline;
     else if ((ptype == ptAvailable) && (show == "online"))
	  return stOnline;
     else if ((ptype == ptUnavailable) && (show.empty()))
	  return stOffline;
     else if ((ptype == ptUnavailable) && (show == "offline"))
	  return stOffline;
     else if (show == "chat")
	  return stChat;
     else if (show == "away")
	  return stAway;
     else if (show == "xa")
	  return stXA;
     else if (show == "dnd")
	  return stDND;
     else if (ptype == ptAvailable) // Maybe we should return stOnline?
	  return stInvalid;         // Only way this could occur is if it's and unknown <show>
     else
	  return stOffline;
}


