
#ifndef PRESENCE_H
#define PRESENCE_H


#include "JaguarX.h"


namespace JaguarX
{


class JPresence
: public JPacket
{
public:
		// 7 Ÿ
		  enum Type {
			  ptSubRequest,   /**< A subscription request. */
			  ptUnsubRequest, /**< An unsubscription request. */
			  ptSubscribed,   /**< Subscription confirmed/allowed/approved. */
			  ptUnsubscribed, /**< Subscription revoked/denied. */
			  ptAvailable,    /**< Available presence (standard online JPresence). */
			  ptUnavailable,  /**< Unavailable JPresence (offline). */
			  ptError,        /**< Error (offline). */
			  ptInvisible     /**< Invisible JPresence, others see you as offline. */
		  };
		  
		  // 7 Ÿ
		  enum Show {
			  stInvalid, /**< Not a valid show type. */
				  stOffline, /**< Show user as offline. */
				  stOnline,  /**< Show user as online. */
				  stChat,    /**< Show user as wanting to chat. */
				  stAway,    /**< Show user as away. */
				  stXA,      /**< Show user as extended away. */
				  stDND      /**< Show user as Do Not Disturb. */
		  };
		  
		  JPresence(const Element& t);
		  JPresence(const string& jid, Type ptype, Show stype = stInvalid, const string& status = "", const string& priority = "0");
		  void setType(JPresence::Type ptype);
		  void setStatus(const string& status);
		  void setShow(JPresence::Show stype);
		  void setPriority(const string& priority);
		  Type          getType()     const;
		  Show          getShow()     const;
		  const string  getStatus()   const;
		  const string  getShow_str() const;
		  int           getPriority() const;
		  
protected:
		  static string translateType(Type ptype);
		  static Type   translateType(const string& ptype);
		  static string translateShow(Show stype);
		  static Show   translateShow(Type ptype, const string& stype);
		  
private:
		  Show   _show;
		  Type   _type;
		  int    _priority;
};

}//end of JaguarX namespace

#endif