

#pragma warning(disable:4786)

#include "JaguarX.h"

using namespace JaguarX;

///////////////////////////////////////////////////////////////////////////
// Helpful predicates
class pred_presence_jid
{
string _jid;
public:
     pred_presence_jid(const JPresence& p)
	  : _jid(p.getFrom()) {}
     pred_presence_jid(const string& s) : _jid(s) {}
     // is there a string function to do a case insensitive compare?
     bool operator()(const JPresence& p) const
	  { return (JID::compare(_jid, p.getFrom()) == 0); }
};

///////////////////////////////////////////////////////////////////////////
class pred_presence_priority
{
int _priority;
public:
     pred_presence_priority(const JPresence& p)
	  : _priority(p.getPriority()){}
     bool operator()(const JPresence& p) const
	  { return p.getPriority() <= _priority; }
};
///////////////////////////////////////////////////////////////////////////

JPresenceDB::JPresenceDB(JSession& s)
     : _Owner(s)
{}

// 񿡼 ãƼ JID õ iterator ãƼ ش.
JPresenceDB::db::const_iterator JPresenceDB::find_or_throw(const string& jid) const
{
     db::const_iterator it = _DB.find(JID::getUserHost(jid));
     if (it != _DB.end())
	  return it;
     else
	  throw XCP_InvalidJID();
}

//  Ѵ.
//   ؾ ϳ?
// khos_remake
void display(const JPresence& p)
{
     cerr << "\tItem: " << p.toString() << endl;
}


void JPresenceDB::insert(const JPresence& p)
{
     // Get a reference to the list (create one if necessary)
	// khos_no
	// map somemap[] ̷  ֳ?  ǹϴ...
     list<JPresence>& l = _DB[JID::getUserHost(p.getFrom())];
     
     // If this JPresence is ptUnavailable, remove it from the cache
	 // ǹ presence ...Ѵ.
     if (p.getType() == JPresence::ptUnavailable ||
	 p.getType() == JPresence::ptError)
     {
	  // Empty list? Then exit..
	  if (l.empty())
	  {
	       _DB.erase(JID::getUserHost(p.getFrom()));
	       return;
	  }
	  //   ..
	  else
	  {
	       // Attempt to find the JPresence packet and erase
	       iterator it = find_if(l.begin(), l.end(), pred_presence_jid(p));
	       if (it != l.end())
		    l.erase(it);
	       // If the list is now empty, remove this entry from the _DB map
	       if (l.empty())
		    _DB.erase(JID::getUserHost(p.getFrom()));
	}
    }
    // Otherwise, insert/update 
    else 
    {
	// If this list is empty, insert this presence and be done with it
	if (l.empty())
	  {
	       l.push_back(p);
	  }
	  // Otherwise insert into the list
	  else
	  {
	       // Identify any existing items w/ this JID 
	       iterator it = find_if(l.begin(), l.end(), pred_presence_jid(p));
	       if (it != l.end())
	       {
		    // If the item found has the same priority, simply update
		    if (it->getPriority() == p.getPriority())
		    {
			 // Replace and exit
			 *it = p;
			 return;
		    }
		    // Otherwise erase the element 
		    else
			 l.erase(it);
	       }
	       // Now identify the insertion point for this element
	       it = find_if(l.begin(), l.end(), pred_presence_priority(p));
	       l.insert(it, p);
	  }
     }
}

void JPresenceDB::remove(const string& jid)
{
     list<JPresence>& l = _DB[JID::getUserHost(jid)];
     
     // Empty list? Then exit..
     if (l.empty())
     {
	  _DB.erase(JID::getUserHost(jid));
	  return;
     }
     else
     {
	  // Attempt to find the presence packet and erase
	  iterator it = find_if(l.begin(), l.end(), pred_presence_jid(jid));
	  if (it != l.end())
	       l.erase(it);
	  // If the list is now empty, remove this entry from the _DB map
	  if (l.empty())
	       _DB.erase(JID::getUserHost(jid));
     }
}

// Ѵ.??
// khos_no
JPresenceDB::range JPresenceDB::equal_range(const string& jid) const
{
    db::const_iterator it = find_or_throw(jid);
    return make_pair(it->second.begin(), it->second.end());
}

// khos_no
JPresence JPresenceDB::findExact(const string& jid) const
{
     JPresenceDB::db::const_iterator it = find_or_throw(jid);

     // If the list is empty, throw an exception
     if (it->second.begin() == it->second.end())
	  throw XCP_InvalidJID();

     // otherwise check for a resource
     string::size_type i = jid.find("/");
     if (i != string::npos)
     {
	  list<JPresence> l= it->second;

	  //return matching resource
	  // Identify any existing items w/ this JID 
	  iterator it2 = find_if(l.begin(), l.end(), pred_presence_jid(jid));

	  // if we didn't find an entry, throw an exception
	  if (it2 == l.end())
	       throw XCP_InvalidJID();

	  // return presence matching the full JID
	  return *it2;
     }
     else
     {
	  //return the first item in the list
	  return *find_or_throw(jid)->second.begin();
     }
}

JPresenceDB::const_iterator JPresenceDB::find(const string& jid) const
{
     JPresenceDB::db::const_iterator it = find_or_throw(jid);
     // If the list is empty, throw an exception, otherwise return the first
     // item in the list
     if (it->second.begin() != it->second.end())
	  return find_or_throw(jid)->second.begin();
     else
	  throw XCP_InvalidJID();
}

bool JPresenceDB::contains(const string& jid) const
{
     return (_DB.find(JID::getUserHost(jid)) != _DB.end());
}

bool JPresenceDB::available(const string& jid) const
{
     db::const_iterator it = _DB.find(JID::getUserHost(jid));
     // it->second is a list that could be empty so we need to check
     // if there is actually an item in the list before we call getType
     // on the first iitem
     if (it != _DB.end() && it->second.begin() != it->second.end())
	  return (it->second.begin()->getType() == JPresence::ptAvailable);
     else
	  return false;
}

void JPresenceDB::clear()
{
     // Erase all entries from the DB
     _DB.clear();
}
