
#ifndef SESSION_H
#define SESSION_H

#include "JaguarX.h"


//#define ElementCallbackFunc		void(*func)(const Element&)


namespace JaguarX
{

/*
 	
class SigFuncElement 
{
	void (*func)(JSession * inst,const Element& t);
public:
	SigFuncElement(){};
	~SigFuncElement(){};
	void exec(JSession * inst,const Element& t);
	void join(void(*f)(JSession * inst,const Element&t));

};

*/
/*
class JSig
{
public:
	JSig(){};
	~JSig(){};
	OnAuthTypeReceived(const Element &t);
	IQHandler_Auth(const Element &t);
	IQHandler_CreateUser(const Element &t);
};
*/

class JRoster
{
public:
	
//////////////////////////////////////////////////////////////////////////
	enum Subscription {
		rsNone,  /**< Neither of you can see the other's JPresence. */
			rsTo,    /**< You can see their JPresence, but they cannot see yours. */
			rsFrom,  /**< You cannot see their JPresence, but they can see yours. */
			rsBoth,  /**< Both of you can see each other's JPresence. */
			rsRemove /**< This JRoster Item is being removed. */
	};
//////////////////////////////////////////////////////////////////////////

class Item {
public:
	Item(const Element& t);
	Item(JRoster& r, const Element& t);
	Item(const string& jid, const string& nickname);
	~Item();
public:
	void addToGroup(const string& group);
	void delFromGroup(const string& group);
	void clearGroups();
	void setNickname(const string& nickname);
	void setJID(const string& jid);
	bool             isAvailable() const;
	string           getNickname() const;
	string           getJID() const;
	Subscription     getSubsType() const;
	bool             isPending() const;
	
	// Group pseudo-container ops/iterators
	typedef set<string>::const_iterator iterator;
	iterator begin() const { return _groups.begin(); }
	iterator end()   const { return _groups.end(); }
	bool     empty() const { return _groups.empty(); }
	
protected:
	// Update handlers
	void update(JRoster& r, const string& jid, const JPresence& p, JPresence::Type prev_type);
	bool update(const Element& t);
	bool update(JRoster& r, const Element& t);
private:
	friend class JRoster;
	int          _rescnt;
	set<string>  _groups;
	Subscription _type;
	bool         _pending;
	string       _nickname;
	string       _jid;
};
//////////////////////////////////////////////////////////////////////////
		  
	class XCP_InvalidJID : public XCP{};
	
//////////////////////////////////////////////////////////////////////////
	
	typedef map<string, Item, jutil::CaseInsensitiveCmp> ItemMap;
	
	// Non-const iterators
	typedef jutil::ValueIterator<ItemMap::iterator, Item > iterator;
	iterator begin() { return _items.begin(); }
	iterator end() { return _items.end(); }
	
	// Const iterators
	typedef jutil::ValueIterator<ItemMap::const_iterator, const Item> const_iterator;
	const_iterator begin() const { return _items.begin(); }
	const_iterator end() const { return _items.end(); }

	JRoster(JSession& s);
	JSession& getSession() { return _owner; }
	const JRoster::Item& operator[](const string& jid) const;
	JRoster& operator<<(const Item& i) { update(i); return *this; }

public:
	void reset();
	
	bool containsJID(const string& jid) const;
	
	void update(const Element& t);
	void update(const JPresence& p, JPresence::Type prev_type); 
	void update(const Item& i);	   // JRoster item add/modify
	
	void deleteUser(const string& jid); /* Remove the user w/ JID */
	void fetch() const;                 /* Retrieve JRoster from server */             
	
	// Translation
	static string translateS10N(Subscription stype);
	static Subscription translateS10N(const string& stype);
	static string filterJID(const string& jid);
	
	// Item/Group access -- HACK
	const map<string, set<string> >& getGroups() const { return _groups;} 
public:
	//Signal0<void, Marshal<void> >    evtRefresh;
	//Signal3<void, const string&, bool, JPresence::Type, Marshal<void> > evtPresence;

	void(*evtRefresh)(void);
	void(*evtPresence)(const string&,bool,JPresence::Type);

private:
	friend class Item;
	void mergeItemGroups(const string& itemjid, const set<string>& oldgrp, const set<string>& newgrp);
	void removeItemFromGroup(const string& group, const string& jid);
	void removeItemFromAllGroups(const Item& item);
	void deleteAgent(const Element& iq);

	friend void Callback_deleteAgent(JSig * inst,const Element & iq);

	SigFuncElement _SigdeleteAgent;
	
	ItemMap                   _items;
	map<string, set<string> > _groups;
	JSession&                  _owner;
};



//////////////////////////////////////////////////////////////////////////
class JSession : 
public ElementStreamEventListener, public ElementStream , public JSig
{

public:
	enum AuthType
	{ 
		atPlaintextAuth, /**< Send the password as plaintext (plaintext). */
			atDigestAuth,    /**< Send a hash of the password (digest). */
			atAutoAuth,      /**< Automatically determine authorization type, defaulting to the most secure. */
			at0kAuth         /**< Do not send any password information over the wire (zero knowledge). */
	};
	
	/**
	* The connection state.
	*/
	enum ConnectionState
	{
		csNotConnected, /**< Not connected. */
			csCreateUser,   /**< Create a new user. */
			csAuthReq,      /**< Auhtorization request. */
			csAwaitingAuth, /**< Awaiting authorization. */
			csConnected     /**< Connected. */
	};
	
	JSession();
	virtual ~JSession();
	JSession& operator>>(const char* buffer) { push(buffer, strlen(buffer)); return *this; } 
	JSession& operator<<(const JPacket& p) { evtTransmitPacket(p); evtTransmitXML(p.toString().c_str()); return *this;}
	JSession& operator<<(const char* buffer) { evtTransmitXML(buffer); return *this;}
	

public:
	void connect(const string& server, AuthType atype, 
		const string& username, const string& resource, const string& password, 
		bool newuser = false);
	
	bool disconnect();
	
	ConnectionState getConnState() { return _ConnState; }
	virtual void push(const char* data, int datasz);
//	void registerIQ(const string& id, ElementCallbackFunc f);
//	void queryNamespace(const string& nspace, ElementCallbackFunc f, const string& to = "");
	void registerIQ(const string& id, SigFuncElement f);
	void queryNamespace(const string& nspace,SigFuncElement f, const string & to = "");

/*
	const JRoster& JRoster() const;
	JRoster&       JRoster();
	const JPresenceDB& JPresenceDB() const;
	JPresenceDB&       JPresenceDB();
*/
	const JRoster& getRosterConst() const;
	JRoster& getRoster();
	const JPresenceDB& getPresenceDBConst() const;
	JPresenceDB& getPresenceDB();

	AuthType      getAuthType() const;
	const string& getUserName() const;
	
	string getNextID();
	string getDigest();
	
public:
	void(*evtXMLParserError)(int,const string&);
	void(*evtTransmitXML)(const char *);
	void(*evtRecvXML)(const char *);
	void(*evtTransmitPacket)(const JPacket&);
	void(*evtConnected)(const Element&);
	void(*evtDisconnected)(void);
	void(*evtMessage)(const JMessage&);
	void(*evtIQ)(const Element&);
	void(*evtMyPresence)(const JPresence&);
	void(*evtPresence)(const JPresence&,JPresence::Type);
	void(*evtPresenceRequest)(const JPresence&);
	void(*evtUnknownPacket)(const Element&);
	void(*evtAuthError)(int,const char*);
	void(*evtOnRoster)(void);
	void(*evtOnVersion)(string&,string&,string&);
	void(*evtOnLast)(string&);
	void(*evtOnTime)(string&,string&);

public:
		  // Authentication handler
		  void authenticate();
		  void OnAuthTypeReceived(const Element& t);
		  void sendLogin(JSession::AuthType atype, const Element* squery);
		  
		  // ElementStream events
		  virtual void onDocumentStart(Element* t);
		  virtual void onElement(Element* t);
		  virtual void onCDATA(CDATA* c);
		  virtual void onDocumentEnd();
		  
		  // Basic packet handlers
		  void handleMessage(Element& t);
		  void handlePresence(Element& t);
		  void handleIQ(Element& t);

private:
		  SigFuncElement _SigOnAuthTypeReceived;
		  SigFuncElement _SigIQHandler_CreateUser;
		  SigFuncElement _SigIQHandler_Auth;

public:
		  friend void Callback_OnAuthTypeReceived(JSig * inst, const Element& t);
		  friend void Callback_IQHandler_CreateUser(JSig * inst, const Element& t);
		  friend void Callback_IQHandler_Auth(JSig * inst, const Element& t);
private:
		  // Values
		  string          _ServerID;
		  string          _Username;
		  string          _Password;
		  string          _Resource;
		  string          _SessionID;
		  long            _ID;
		  int				_CallNum;
		  // Internal JRoster & JPresence db structures
		  JRoster          _Roster;
		  JPresenceDB      _PDB;

		  // Enumeration values (states)
		  AuthType        _AuthType;
		  ConnectionState _ConnState;
		  // Stream header 
		  Element*            _StreamElement;
		  // Wether or not we have received the starting stream tag
		  bool            _StreamStart;
		  // Structures
//		  multimap<string, ElementCallbackFunc> _Callbacks;			 /* IQ callback funcs */
		  multimap<string, SigFuncElement> _Callbacks;

public:		  
		  // Internal IQ handlers
		  void IQHandler_Auth(const Element& t);
		  void IQHandler_CreateUser(const Element& t);
};    

}//end of JaguarX namespace

#endif