// SHPViewerView.cpp : implementation of the CSHPViewerView class
//
// Coded by Scene, Sanghee(shshin@gaia3d.com)
// 2001-11-14
// Ver 0.5
// Copyright(c) 2001 Gaia3D Inc & Scene, Sanghee
//
///////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "SHPViewer.h"

#include "SHPViewerDoc.h"
#include "SHPViewerView.h"

#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSHPViewerView

IMPLEMENT_DYNCREATE(CSHPViewerView, CView)

BEGIN_MESSAGE_MAP(CSHPViewerView, CView)
	//{{AFX_MSG_MAP(CSHPViewerView)
	ON_COMMAND(IDM_ZOOM_IN, OnZoomIn)
	ON_UPDATE_COMMAND_UI(IDM_ZOOM_IN, OnUpdateZoomIn)
	ON_COMMAND(IDM_ZOOM_OUT, OnZoomOut)
	ON_UPDATE_COMMAND_UI(IDM_ZOOM_OUT, OnUpdateZoomOut)
	ON_COMMAND(IDM_ZOOM_ALL, OnZoomAll)
	ON_COMMAND(IDM_CENTER_MOVE, OnCenterMove)
	ON_UPDATE_COMMAND_UI(IDM_CENTER_MOVE, OnUpdateCenterMove)
	ON_COMMAND(IDM_PANNING, OnPanning)
	ON_UPDATE_COMMAND_UI(IDM_PANNING, OnUpdatePanning)
	ON_COMMAND(IDM_ZOOM_WINDOW, OnZoomWindow)
	ON_UPDATE_COMMAND_UI(IDM_ZOOM_WINDOW, OnUpdateZoomWindow)
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_SETCURSOR()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(IDM_FILE_CLOSE, OnFileClose)
	ON_COMMAND(IDM_CHANGE_COLOR, OnChangeColor)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSHPViewerView construction/destruction

CSHPViewerView::CSHPViewerView()
{
	m_bOpenSHP = false;
	m_lfZoomFactor = 1.0;
	m_geoCenterPoint.x = m_geoCenterPoint.y = 0.0;
	m_scrCenterPoint.x = m_scrCenterPoint.y = 0;
}

CSHPViewerView::~CSHPViewerView()
{

}

BOOL CSHPViewerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSHPViewerView drawing

void CSHPViewerView::OnDraw(CDC* pDC)
{
	CSHPViewerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect rect;
	GetClientRect(&rect);	
	
	// Ⱦ ..
	double	lfRatio_X = fabs((rect.Width()) / (m_MBR.xmax - m_MBR.xmin));
	double	lfRatio_Y = fabs((rect.Height()) / (m_MBR.ymax - m_MBR.ymin));

	//  ...
	if(lfRatio_X < lfRatio_Y)
		m_lfRatio = lfRatio_X;
	else
		m_lfRatio = lfRatio_Y;

	m_scrCenterPoint.x = (rect.right-rect.left)/2;
	m_scrCenterPoint.y = (rect.bottom-rect.top)/2;

	//  double buffering
	CDC mem_dc;
	CDC* pDrawDC = pDC;
	CBitmap bitmap;
	CBitmap* pOldBitmap;

	if (mem_dc.CreateCompatibleDC(pDC))
	{
		if (bitmap.CreateCompatibleBitmap(pDC, rect.Width(), rect.Height()))
		{
			pDrawDC = &mem_dc;
			pOldBitmap = mem_dc.SelectObject(&bitmap);
		}
	}
	pDrawDC->FillSolidRect(&rect, RGB(255, 255, 255));

	if(m_bOpenSHP)
		m_SHP.Draw(pDrawDC, m_lfZoomFactor, m_geoCenterPoint, rect, m_lfRatio);

	//Swap buffers
	if (pDrawDC != pDC)
	{
		pDC->BitBlt(rect.left, rect.top, rect.Width(), rect.Height(),
			&mem_dc,0 ,0 , SRCCOPY);
		mem_dc.SelectObject(pOldBitmap);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSHPViewerView printing

BOOL CSHPViewerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSHPViewerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CSHPViewerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CSHPViewerView diagnostics

#ifdef _DEBUG
void CSHPViewerView::AssertValid() const
{
	CView::AssertValid();
}

void CSHPViewerView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSHPViewerDoc* CSHPViewerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSHPViewerDoc)));
	return (CSHPViewerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSHPViewerView message handlers
void CSHPViewerView::OnZoomIn() 
{
	(m_eManipulationMode==ZoomIn) ? m_eManipulationMode = None : m_eManipulationMode = ZoomIn;
}

void CSHPViewerView::OnUpdateZoomIn(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_eManipulationMode==ZoomIn));
}

void CSHPViewerView::OnZoomOut() 
{
	(m_eManipulationMode==ZoomOut) ? m_eManipulationMode = None : m_eManipulationMode = ZoomOut;
}

void CSHPViewerView::OnUpdateZoomOut(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_eManipulationMode==ZoomOut));
}

void CSHPViewerView::OnCenterMove() 
{
	(m_eManipulationMode==CenterMove) ? m_eManipulationMode = None : m_eManipulationMode = CenterMove;
}

void CSHPViewerView::OnUpdateCenterMove(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_eManipulationMode==CenterMove));
}

void CSHPViewerView::OnPanning() 
{
	(m_eManipulationMode==Panning) ? m_eManipulationMode = None : m_eManipulationMode = Panning;
}

void CSHPViewerView::OnUpdatePanning(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_eManipulationMode==Panning));
}

void CSHPViewerView::OnZoomWindow() 
{
	(m_eManipulationMode==ZoomWindow) ? m_eManipulationMode = None : m_eManipulationMode = ZoomWindow;
}

void CSHPViewerView::OnUpdateZoomWindow(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((m_eManipulationMode==ZoomWindow));
}

void CSHPViewerView::OnZoomAll() 
{
	//߽ǥ ϱ
	m_geoCenterPoint.x = (m_MBR.xmin + m_MBR.xmax)/2;
	m_geoCenterPoint.y = (m_MBR.ymin + m_MBR.ymax)/2;
	
	m_lfZoomFactor = 1.0;

	Invalidate();
}

void CSHPViewerView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	m_scrCenterPoint.x = cx/2;
	m_scrCenterPoint.y = cy/2;
}

void CSHPViewerView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	switch(m_eManipulationMode)
	{
	case ZoomIn:
		m_lfZoomFactor *= 1.6;
		Invalidate();
		break;

	case ZoomOut:
		m_lfZoomFactor *= 0.6;
		Invalidate();
		break;
	
	case CenterMove:
		m_geoCenterPoint = DeviceToWorld(point);
		Invalidate();
		break;
	
	case ZoomWindow:
	case Panning:
		m_ptZoomWindowLT = m_ptZoomWindowBR = point;
		SetCapture();
		break;
	}
	CView::OnLButtonDown(nFlags, point);
}

void CSHPViewerView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CRect	rect;			// ȭũ⸦ ޾ƿ 

	switch(m_eManipulationMode)
	{
	case ZoomWindow:  // Ȯ 
		GeoPoint	tmpPoint1, tmpPoint2;
		double ZoomWidth, ZoomHeight;
		double ZoomRatio;

		ReleaseCapture();  // 콺 ư ̴  콺 ̺Ʈ ұ Windows ش. 
							//  ̷ Լ SetCapture().. OnLButtonDown ã  
		m_ptZoomWindowBR = point;
		tmpPoint1 = DeviceToWorld(m_ptZoomWindowLT);
		tmpPoint2 = DeviceToWorld(m_ptZoomWindowBR);
		
		// ǥ迡 Ȯâ ũ⸦ Ѵ.
		ZoomWidth  = fabs(tmpPoint1.x - tmpPoint2.x);
		ZoomHeight = fabs(tmpPoint1.y - tmpPoint2.y);

		GetClientRect(&rect);  // ȭ ũ⸦ ޾ƿ´.
		
		if(ZoomWidth >= ZoomHeight)
			ZoomRatio = rect.Width()/ZoomWidth;	//Zoom ũ⸦ Ѵ.
		else
			ZoomRatio = rect.Height()/ZoomHeight;
		
		m_lfZoomFactor = ZoomRatio/m_lfRatio;  // ZoomIn, ZoomOut  m_fZoomRatio  óѴ. 
		
		m_geoCenterPoint.x = (tmpPoint1.x + tmpPoint2.x) / 2;  // ߽ ǥ  ش.
		m_geoCenterPoint.y = (tmpPoint1.y + tmpPoint2.y) / 2;  // ߽ ǥ Ѵ.

		Invalidate();

		break;
	
	case Panning:  // д  ...
		ReleaseCapture();  // 콺 ̺Ʈ   ְ..
		break;
	}
	CView::OnLButtonUp(nFlags, point);
}

void CSHPViewerView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CRect	tmpRect;
	GetClientRect(&tmpRect);

	m_scrCenterPoint.x = tmpRect.Width() / 2;
	m_scrCenterPoint.y = tmpRect.Height() / 2;

	char buffer[30];
	int i;
	GeoPoint	tmp;

	CStatusBar *pStatusBar = GetStatusBar(); // ¹ ͸ ..
    if (!pStatusBar) return;

	
    // Draw a Geo-Coordinate of mouse point
	tmp=DeviceToWorld(point);
	i = sprintf(buffer, "X=%.2lf   ", tmp.x);
	i+= sprintf(buffer+i, "Y=%.2lf", tmp.y);

    if (::IsWindow(m_hWnd) && IsWindowVisible()) 
	{
        pStatusBar->SetWindowText(buffer);
        pStatusBar->UpdateWindow();
	}

	// Mouse manipulation mode
	switch(m_eManipulationMode)
	{
	case ZoomWindow:
		if(nFlags & MK_LBUTTON)
		{
			CClientDC	dc(this);
			CPen	cPen, *oldPen;
			
			cPen.CreatePen(PS_DOT, 1, RGB(0,0,0));
			oldPen = (CPen*)dc.SelectObject(&cPen);
			dc.SetROP2(R2_XORPEN);
			
			// Window .
			dc.MoveTo(m_ptZoomWindowLT);
			dc.LineTo(m_ptZoomWindowLT.x, m_ptZoomWindowBR.y);
			dc.LineTo(m_ptZoomWindowBR);
			dc.LineTo(m_ptZoomWindowBR.x, m_ptZoomWindowLT.y);
			dc.LineTo(m_ptZoomWindowLT);
			// ο window ׸.
			dc.MoveTo(m_ptZoomWindowLT);
			dc.LineTo(m_ptZoomWindowLT.x, point.y);
			dc.LineTo(point);
			dc.LineTo(point.x, m_ptZoomWindowLT.y);
			dc.LineTo(m_ptZoomWindowLT);

			m_ptZoomWindowBR = point;
			dc.SelectObject(oldPen);
		}
		break;

	case Panning:
		if(nFlags & MK_LBUTTON)
		{
			double dX = (m_ptZoomWindowBR.x - point.x) ;
			double dY = (m_ptZoomWindowBR.y - point.y) ;

			m_geoCenterPoint.x += dX / (m_lfRatio * m_lfZoomFactor );
			m_geoCenterPoint.y -= dY / (m_lfRatio * m_lfZoomFactor );
			
			m_ptZoomWindowBR = point;

			InvalidateRect(NULL);
		}
	}
	CView::OnMouseMove(nFlags, point);
}

CPoint CSHPViewerView::WorldToDevice(const GeoPoint& _geoPoint)
{
	CPoint	tmpPoint;

	tmpPoint.x = long(m_lfRatio * (_geoPoint.x - m_geoCenterPoint.x) * m_lfZoomFactor + m_scrCenterPoint.x);
	tmpPoint.y = long(m_lfRatio * (m_geoCenterPoint.y - _geoPoint.y) * m_lfZoomFactor + m_scrCenterPoint.y);

	return tmpPoint;
}

GeoPoint CSHPViewerView::DeviceToWorld(const CPoint& _scrPoint)
{
	GeoPoint	tmpGeoPoint;

	tmpGeoPoint.x = m_geoCenterPoint.x + (_scrPoint.x - m_scrCenterPoint.x) / (m_lfZoomFactor * m_lfRatio);
	tmpGeoPoint.y = m_geoCenterPoint.y - (_scrPoint.y - m_scrCenterPoint.y) / (m_lfZoomFactor * m_lfRatio);

	return tmpGeoPoint;
}

BOOL CSHPViewerView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	HCURSOR	hCursor;

	switch(m_eManipulationMode)
	{
	case ZoomIn:		// ZoomIn, ZoomWindow  Ŀ 
	case ZoomWindow:
		hCursor = AfxGetApp()->LoadCursor(IDC_ZOOM_IN);
		SetCursor(hCursor);
		return true;

	case ZoomOut:		// ZoomOut Ŀ..
		hCursor = AfxGetApp()->LoadCursor(IDC_ZOOM_OUT);
		SetCursor(hCursor);
		return true;

	case Panning:		// д Ŀ..
		hCursor = AfxGetApp()->LoadCursor(IDC_PANNING);
		SetCursor(hCursor);
		return true;

	case CenterMove:	// ̵߽ Ŀ..
		hCursor = AfxGetApp()->LoadCursor(IDC_ZOOM_CENTER);
		SetCursor(hCursor);
		return true;
	}

	return CView::OnSetCursor(pWnd, nHitTest, message);
}

BOOL CSHPViewerView::OnEraseBkgnd(CDC* pDC) 
{
//	return CView::OnEraseBkgnd(pDC);
	return false;
}

CStatusBar* CSHPViewerView::GetStatusBar()
{
    CFrameWnd *pFrame = (CFrameWnd*)AfxGetMainWnd();
    if (!pFrame || !pFrame->IsKindOf(RUNTIME_CLASS(CFrameWnd))) 
        return NULL;

    CStatusBar* pBar = (CStatusBar*) pFrame->GetMessageBar();
    if (!pBar || !pBar->IsKindOf(RUNTIME_CLASS(CStatusBar))) 
        return NULL;

    return pBar;
}

void CSHPViewerView::OnFileOpen() 
{
	CRect	tmpRect;
	GetClientRect(tmpRect);

	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY, 
				"SHP Files (*.shp)|*.shp|All Files (*.*)|*.*|", NULL);

	// SHP Loading
	if(dlg.DoModal() != IDOK)
		return;

	//  ̹  ݴ´.
	if(m_bOpenSHP)
		m_SHP.Close();

	if(!m_SHP.Open(dlg.GetPathName()))
		return;

	ReMBR(m_SHP.m_MBR);
	m_bOpenSHP = true;

	// ȭ鿡 Ѹ ؼ ....
	m_geoCenterPoint.x = (m_MBR.xmax + m_MBR.xmin) / 2;
	m_geoCenterPoint.y = (m_MBR.ymax + m_MBR.ymin) / 2;
	m_lfZoomFactor = 1.;
	m_lfRatio = tmpRect.Height() / m_MBR.ymax;

	Invalidate();
}

void CSHPViewerView::OnFileClose() 
{
	m_bOpenSHP = !m_SHP.Close();

	Invalidate();
}

void CSHPViewerView::OnChangeColor() 
{
	CColorDialog	dlg;
	
	if(dlg.DoModal()==IDOK)
	{
		m_SHP.m_lineColor = dlg.GetColor();
		Invalidate();
	}
	else
		return;
}

void CSHPViewerView::ReMBR(const MBR& _mbr)
{
	// only for multiple SHP
	/*	
		if(_mbr.xmax>=m_MBR.xmax) m_MBR.xmax=_mbr.xmax;
		if(_mbr.xmin<=m_MBR.xmin) m_MBR.xmin=_mbr.xmin;
		if(_mbr.ymax>=m_MBR.ymax) m_MBR.ymax=_mbr.ymax;
		if(_mbr.ymin<=m_MBR.ymin) m_MBR.ymin=_mbr.ymin;
	*/
	m_MBR = _mbr;

	m_geoCenterPoint.x = (m_MBR.xmax + m_MBR.xmin) / 2;
	m_geoCenterPoint.y = (m_MBR.ymax + m_MBR.ymin) / 2;
}
