// Shape.cpp: implementation of the CShape class.
//
// Coded by Scene, Sanghee(shshin@gaia3d.com)
// 2001-11-14
// Ver 0.5
// Copyright(c) 2001 Gaia3D Inc & Scene, Sanghee
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Shape.h"

#include "time.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CShape::CShape()
{
	m_bOpen = FALSE;
	
	m_MBR.xmax = m_MBR.ymax = m_MBR.xmin = m_MBR.ymin = 0.;
	
	m_nShapeType = 0;

	m_pSHPPoints = NULL;
	m_pSHPPolyObjects = NULL;

	m_lineColor = RGB(0, 0, 0);
	m_lineWidth = 0;
	m_lineType = 0;
}

CShape::~CShape()
{
	Close();
}

void CShape::SwapWord(int _length, void* _pWord)   
{
	int		i;
    unsigned char	temp;

    for( i=0; i< _length/2; i++ )
    {
		temp = ((unsigned char *) _pWord)[i];
		((unsigned char *) _pWord)[i] = ((unsigned char *) _pWord)[_length-i-1];
		((unsigned char *) _pWord)[_length-i-1] = temp;
    }
}

bool CShape::Open(CString _filename)
{
	FILE *fpSHP, *fpSHX;
	int tmp, i, j;
	
	m_strSHPPath = _filename;	//SHP file path

	tmp = _filename.GetLength()-4;
	m_strSHXPath = _filename.Mid(0, tmp) + ".shx"; //SHX file path

	//open SHP/SHX file
	fpSHP = fopen(m_strSHPPath, "rb");
	fpSHX = fopen(m_strSHXPath, "rb");

	if(fpSHP == NULL || fpSHX == NULL)
	{
		AfxMessageBox("SHP File Error!!", IDOK | MB_ICONSTOP);
		return false;
	}

	/* release memory of exsiting one */
	if(m_bOpen)
		Close();

	/* Now read a Header of SHP/SHX */
	// Get File size & Number of records
	fseek(fpSHX, 24L, SEEK_SET);
	fread(&tmp, 4, 1, fpSHX);
	SwapWord(4, &tmp);	// Byte Order Changing
	
	int m_nFileSize = tmp * 2;	// get file size
	m_nRecords = (m_nFileSize - 100)/8;	//get a number of records

	// get shp type
	fseek(fpSHP, 32L, SEEK_SET);
	fread(&m_nShapeType, 4, 1, fpSHP);
	
	// Check a invalid shp type
	if(!(m_nShapeType == 1 || m_nShapeType == 3 || m_nShapeType == 5 || m_nShapeType == 8)) 
	{
		AfxMessageBox("Invalid SHP Type.\nThis program only support Point, Arc, Polygon & MultiPoint SHP Type.", IDOK | MB_ICONSTOP);
		return false;
	}

	// get a MBR
	fread(&m_MBR, sizeof(MBR), 1, fpSHP);

	// real routine of feature reading(point, arc, polygon, multipoint)
	switch(m_nShapeType)
	{
	// Ʈ оδ.
	case SHPT_POINT:
		m_pSHPPoints = new GeoPoint[m_nRecords];

		for(i = 0; i < m_nRecords; ++i)
		{
			fseek(fpSHX, 100 + i * 8, SEEK_SET);
			fread(&tmp, 4, 1, fpSHX);
			SwapWord(4, &tmp);
			tmp=tmp * 2;

			fseek(fpSHP, tmp + 12, SEEK_SET);
			fread((GeoPoint*)&m_pSHPPoints[i], 16, 1, fpSHP);
		}
		break;

	// ΰ  оδ.
	case SHPT_ARC :
	case SHPT_POLYGON:
		m_pSHPPolyObjects = new PolyObject[m_nRecords];

		for(i=0;i<m_nRecords;++i)
		{
			fseek(fpSHX, 100+i*8, SEEK_SET);
			fread(&tmp, 4, 1, fpSHX);
			SwapWord(4, &tmp);
			tmp=tmp*2;

			fseek(fpSHP, tmp+12, SEEK_SET);
			fread((MBR*)&m_pSHPPolyObjects[i].m_MBR, 32, 1, fpSHP);	//get MBR
			fread(&m_pSHPPolyObjects[i].m_nNumParts, 4, 1, fpSHP);	//get parts number
			fread(&m_pSHPPolyObjects[i].m_nNumPoints, 4, 1, fpSHP);  //get points number

			m_pSHPPolyObjects[i].m_pParts = new int[m_pSHPPolyObjects[i].m_nNumParts];
			m_pSHPPolyObjects[i].m_pPoints = new GeoPoint[m_pSHPPolyObjects[i].m_nNumPoints];

			for(j=0;j<m_pSHPPolyObjects[i].m_nNumParts;++j)
				fread(&m_pSHPPolyObjects[i].m_pParts[j], 4, 1, fpSHP);

			for(j=0;j<m_pSHPPolyObjects[i].m_nNumPoints;++j)
				fread(&m_pSHPPolyObjects[i].m_pPoints[j], 16, 1, fpSHP);
		}
		break;
	
	//  оδ.
	case SHPT_MULTIPOINT:
		m_pSHPPolyObjects = new PolyObject[m_nRecords];

		for(i=0;i<m_nRecords;++i)
		{
			fseek(fpSHX, 100+i*8, SEEK_SET);
			fread(&tmp, 4, 1, fpSHX);
			SwapWord(4, &tmp);
			tmp=tmp*2;

			fseek(fpSHP, tmp+12, SEEK_SET);
			fread((MBR*)&m_pSHPPolyObjects[i].m_MBR, 32, 1, fpSHP);	//get  MBR
			fread(&m_pSHPPolyObjects[i].m_nNumPoints, 4, 1, fpSHP);	// get points number

			m_pSHPPolyObjects[i].m_pPoints = new GeoPoint[m_pSHPPolyObjects[i].m_nNumPoints];

			for(j=0;j<m_pSHPPolyObjects[i].m_nNumPoints;++j)
				fread(&m_pSHPPolyObjects[i].m_pPoints[j], 16, 1, fpSHP);
		}
		break;
	}
	// SHP  ϰ ҴѴ.
	srand((unsigned)time(NULL));	// Լ ʱȭ..
	m_lineColor = RGB(rand()%255, rand()%255, rand()%255); //SHP ..

	fclose(fpSHX);
	fclose(fpSHP);

	m_bOpen = true;
	return true;
}

void CShape::Draw(CDC* _pDC, double _ZoomFactor, GeoPoint _ptMapCenter, CRect _rctScreen, double _lfRatio)
{
	m_lfZoomFactor = _ZoomFactor;
	m_geoCenterPoint = _ptMapCenter;
	m_scrCenterPoint.x = int(_rctScreen.Width()/2);
	m_scrCenterPoint.y = int(_rctScreen.Height()/2);
	m_lfMainRatio = _lfRatio;

	switch(m_nShapeType)
	{
	case SHPT_POINT:		//point drawing
		DrawPoint(_pDC);
		break;

	case SHPT_ARC:			//arc drawing
		DrawArc(_pDC);
		break;

	case SHPT_POLYGON:		//polygon drawing
		DrawPolygon(_pDC);
		break;

	case SHPT_MULTIPOINT:	//multipoint drawing
		DrawMultiPoint(_pDC);
		break;
	}
}

CPoint CShape::WorldToDevice(const GeoPoint& _geoPoint, double _lfRatio)
{
	CPoint	tmpPoint;

	tmpPoint.x = long(_lfRatio * (_geoPoint.x - m_geoCenterPoint.x) * m_lfZoomFactor + m_scrCenterPoint.x);
	tmpPoint.y = long(_lfRatio * (m_geoCenterPoint.y - _geoPoint.y) * m_lfZoomFactor + m_scrCenterPoint.y);

	return tmpPoint;
}

GeoPoint CShape::DeviceToWorld(const CPoint& _point, double _lfRatio)
{
	GeoPoint	tmpGeoPoint;

	tmpGeoPoint.x = m_geoCenterPoint.x + (_point.x - m_scrCenterPoint.x) / (m_lfZoomFactor * _lfRatio);
	tmpGeoPoint.y = m_geoCenterPoint.y - (_point.y - m_scrCenterPoint.y) / (m_lfZoomFactor * _lfRatio);

	return tmpGeoPoint;
}

// Ʈ ׸.
void CShape::DrawPoint(CDC* _pDC)
{
	CPen pen;
	CBrush	brush;

	CPoint	tmpPoint;

	int savedDC = _pDC->SaveDC();

	pen.CreatePen(m_lineType, m_lineWidth, m_lineColor);
	brush.CreateSolidBrush(m_lineColor);

	_pDC->SelectObject(&pen);
	_pDC->SelectObject(&brush);

	// Ʈ ŭ 鼭 ȭ鿡 ׸.
	for(int i = 0; i < m_nRecords; ++i)
	{
		tmpPoint = WorldToDevice(m_pSHPPoints[i], m_lfMainRatio);
		_pDC->Ellipse(tmpPoint.x-2, tmpPoint.y-2, tmpPoint.x+2, tmpPoint.y+2);
	}
	_pDC->RestoreDC(savedDC);
}

//  Ȥ ARC  ׸.
void CShape::DrawArc(CDC* _pDC)
{
	CPen	pen;
	
	int savedDC = _pDC->SaveDC();

	pen.CreatePen(m_lineType, m_lineWidth, m_lineColor);
	_pDC->SelectObject(&pen);

	//  ŭ 鼭 ׸.
	for(int i = 0; i < m_nRecords; ++i)
	{
		// ȭ鿡 ׸ Ʈ Ҵϰ..
		CPoint*	pScrPoints = new CPoint[m_pSHPPolyObjects[i].m_nNumPoints];
		for(int j = 0; j < m_pSHPPolyObjects[i].m_nNumPoints; ++j)
			pScrPoints[j] = WorldToDevice(m_pSHPPolyObjects[i].m_pPoints[j], m_lfMainRatio);

		// Ʈ  ޸𸮸 Ҵϰ..
		int* pParts = new int[m_pSHPPolyObjects[i].m_nNumParts];
		for(j = 0; j < m_pSHPPolyObjects[i].m_nNumParts; ++j)
		{
			if(j==m_pSHPPolyObjects[i].m_nNumParts-1)
				pParts[j] = m_pSHPPolyObjects[i].m_nNumPoints - m_pSHPPolyObjects[i].m_pParts[j];
			else
				pParts[j] = m_pSHPPolyObjects[i].m_pParts[j+1] - m_pSHPPolyObjects[i].m_pParts[j];
		}
		// ׸..
		_pDC->PolyPolyline(pScrPoints, (DWORD*)pParts, m_pSHPPolyObjects[i].m_nNumParts);

		delete [] pScrPoints;
		delete [] pParts;
	}
	_pDC->RestoreDC(savedDC);
}

//  ׸.
void CShape::DrawPolygon(CDC* _pDC)
{
	CPen	pen;
	CBrush	brush;
	
	int savedDC = _pDC->SaveDC();
	pen.CreatePen(m_lineType, m_lineWidth, RGB(0,0,0));
	brush.CreateSolidBrush(m_lineColor);
	
	_pDC->SelectObject(&brush);
	_pDC->SelectObject(&pen);

	for(int i=0;i<m_nRecords;++i)
	{
		// ũ Ʈ Ҵϰ..
		CPoint*	pScrPoints = new CPoint[m_pSHPPolyObjects[i].m_nNumPoints];
		for(int j=0;j<m_pSHPPolyObjects[i].m_nNumPoints;++j)
			pScrPoints[j] = WorldToDevice(m_pSHPPolyObjects[i].m_pPoints[j], m_lfMainRatio);

		// Ʈ  ϰ..
		int* pParts = new int[m_pSHPPolyObjects[i].m_nNumParts];
		for(j=0;j<m_pSHPPolyObjects[i].m_nNumParts;++j)
		{
			if(j==m_pSHPPolyObjects[i].m_nNumParts-1)
				pParts[j] = m_pSHPPolyObjects[i].m_nNumPoints - m_pSHPPolyObjects[i].m_pParts[j];
			else
				pParts[j] = m_pSHPPolyObjects[i].m_pParts[j+1] - m_pSHPPolyObjects[i].m_pParts[j];
		}
		// ׸..
		_pDC->PolyPolygon(pScrPoints, pParts, m_pSHPPolyObjects[i].m_nNumParts);

		delete [] pScrPoints;
		delete [] pParts;
	}
	_pDC->RestoreDC(savedDC);
}

//  ׸.
void CShape::DrawMultiPoint(CDC* _pDC)
{
	CPen	pen;
	
	int savedDC = _pDC->SaveDC();

	pen.CreatePen(m_lineType, m_lineWidth, m_lineColor);
	_pDC->SelectObject(&pen);

	for(int i = 0; i < m_nRecords; ++i)
	{
		// ũ  ҴѴ.
		CPoint*	pScrPoints = new CPoint[m_pSHPPolyObjects[i].m_nNumPoints];
		for(int j = 0; j < m_pSHPPolyObjects[i].m_nNumPoints; ++j)
		{
			pScrPoints[j] = WorldToDevice(m_pSHPPolyObjects[i].m_pPoints[j], m_lfMainRatio);
			_pDC->Ellipse(pScrPoints[j].x - 2, pScrPoints[j].y - 2, pScrPoints[j].x + 2, pScrPoints[j].y + 2);
		}
		delete [] pScrPoints;
	}
	_pDC->RestoreDC(savedDC);
}

// Ҵ ޸𸮸 Ѵ.
bool CShape::Close()
{
	if(m_bOpen)
	{
		int i;
		switch(m_nShapeType)
		{
		case SHPT_POINT:
			delete []m_pSHPPoints;
			break;
			
		case SHPT_MULTIPOINT:
			for(i = 0; i < m_nRecords; ++i)
				delete [] m_pSHPPolyObjects[i].m_pPoints;
			break;

		case SHPT_ARC:
		case SHPT_POLYGON:
			for(i = 0; i < m_nRecords; ++i)
			{
				delete [] m_pSHPPolyObjects[i].m_pParts;
				delete [] m_pSHPPolyObjects[i].m_pPoints;
			}
			delete[] m_pSHPPolyObjects;
			break;
		}
	}
	m_bOpen = false;
	return true;
}

