// BaseBar.cpp : implementation file
//
/////////////////////////////////////////////////////////////////////////////
//
//  PROJECT: Outlook Express Style
//	MODULE: Base Bar
//	VERSION: 0.1
//
//		OPERATOR: Jeong Sung-won
//		CONTACT: stupi@neoidea.com
//
//
//		RECORDS:
//			[YYYY-MM-DD] Comment
//
//		CHECK POINT:
//			() NONE
//
//		LAST UPDATE: 2002-05-21
//
//
//	Copylefts (c) 2002 Jeong Sung-won
//	All rights NOT reserved.
//

#include "stdafx.h"
#include "BaseBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



#define BORDERSIZE	5

/////////////////////////////////////////////////////////////////////////////
// CBaseBar

BEGIN_MESSAGE_MAP(CBaseBar, CControlBar)
    //{{AFX_MSG_MAP(CBaseBar)
    ON_WM_WINDOWPOSCHANGED()
    ON_WM_NCPAINT()
    ON_WM_NCCALCSIZE()
	ON_WM_CREATE()
	ON_WM_NCHITTEST()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_PAINT()
	ON_MESSAGE(BPN_CLOSE, OnPaneClose)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// CBaseBar construction/destruction

CBaseBar::CBaseBar()
{
    m_nSize = 0;
	m_nSplit = 0;
    m_nMinSize = 0;
}



CBaseBar::~CBaseBar()
{
}



BOOL CBaseBar::Create(CWnd* pParentWnd, int nSize, int nSplit, int nMinSize /*=100*/, LPCTSTR lpszCaption /*=NULL*/, DWORD dwStyle /*=WS_CHILD|WS_VISIBLE|CBRS_ALIGN_LEFT|CBRS_SIZE_DYNAMIC*/, UINT nID /*=AFX_IDW_DIALOGBAR+1*/)
{
	ASSERT_VALID(pParentWnd);

	m_dwStyle = dwStyle & CBRS_ALL;
	m_nSize = nSize + BORDERSIZE;
	m_nSplit = nSplit;
	m_nMinSize = nMinSize;

	CString strWndClass = AfxRegisterWndClass(CS_DBLCLKS, AfxGetApp()->LoadStandardCursor(IDC_ARROW), NULL, NULL);
    return CWnd::Create(strWndClass, lpszCaption, dwStyle & ~WS_VISIBLE, CRect(0,0,0,0), pParentWnd, nID);
}



int CBaseBar::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if(CControlBar::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_Pane1.Create(NULL, NULL, CRect(0,0,0,0), this);
	m_Pane2.Create(NULL, NULL, CRect(0,0,0,0), this);

	return 0;
}



/////////////////////////////////////////////////////////////////////////////
// CBaseBar drawing

void CBaseBar::OnNcPaint() 
{
	CWindowDC dc(this);

	CBrush brushBackground(GetSysColor(COLOR_3DFACE));
	CPen penHlight(PS_SOLID, 1, GetSysColor(COLOR_3DHIGHLIGHT));
	CPen penShadow(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW));

	CRect rect;
	GetWindowRect(&rect);
	rect.OffsetRect(-rect.TopLeft());

	dc.FillRect(&m_rcBorder, &brushBackground);

	UINT uAlign = GetStyle() & CBRS_ALIGN_ANY;
    switch(uAlign)
    {
	case CBRS_ALIGN_TOP:
		dc.SelectObject(&penHlight);
		dc.MoveTo(rect.right - 1, rect.top);
		dc.LineTo(rect.right - 1, m_rcBorder.top);
		dc.LineTo(rect.left, m_rcBorder.top);
		dc.SelectObject(&penShadow);
		dc.LineTo(rect.left, rect.top);
		dc.LineTo(rect.right - 1, rect.top);
		break;

	case CBRS_ALIGN_BOTTOM:
		dc.SelectObject(&penHlight);
		dc.MoveTo(rect.right - 1, m_rcBorder.bottom - 1);
		dc.LineTo(rect.right - 1, rect.bottom - 1);
		dc.LineTo(rect.left, rect.bottom - 1);
		dc.SelectObject(&penShadow);
		dc.LineTo(rect.left, m_rcBorder.bottom - 1);
		dc.LineTo(rect.right - 1, m_rcBorder.bottom - 1);
		break;

	case CBRS_ALIGN_LEFT:
		dc.SelectObject(&penHlight);
		dc.MoveTo(m_rcBorder.left, rect.top);
		dc.LineTo(m_rcBorder.left, rect.bottom - 1);
		dc.LineTo(rect.left, rect.bottom - 1);
		dc.SelectObject(&penShadow);
		dc.LineTo(rect.left, rect.top);
		dc.LineTo(m_rcBorder.left, rect.top);
		break;

	case CBRS_ALIGN_RIGHT:
		dc.SelectObject(&penHlight);
		dc.MoveTo(rect.right - 1, rect.top);
		dc.LineTo(rect.right - 1, rect.bottom - 1);
		dc.LineTo(m_rcBorder.right - 1, rect.bottom - 1);
		dc.SelectObject(&penShadow);
		dc.LineTo(m_rcBorder.right - 1, rect.top);
		dc.LineTo(rect.right - 1, rect.top);
		break;
    }

	dc.FillRect(&m_rcSplit, &brushBackground);
    switch(uAlign)
    {
	case CBRS_ALIGN_TOP:
		dc.SelectObject(&penHlight);
		dc.MoveTo(m_rcSplit.left, m_rcSplit.top + 1);
		dc.LineTo(m_rcSplit.left, m_rcSplit.bottom - 2);
		dc.SelectObject(&penShadow);
		dc.MoveTo(m_rcSplit.right, m_rcSplit.top + 1);
		dc.LineTo(m_rcSplit.right, m_rcSplit.bottom - 2);
		break;

	case CBRS_ALIGN_BOTTOM:
		dc.SelectObject(&penHlight);
		dc.MoveTo(m_rcSplit.left, m_rcSplit.top + 2);
		dc.LineTo(m_rcSplit.left, m_rcSplit.bottom - 1);
		dc.SelectObject(&penShadow);
		dc.MoveTo(m_rcSplit.right, m_rcSplit.top + 2);
		dc.LineTo(m_rcSplit.right, m_rcSplit.bottom - 1);
		break;

	case CBRS_ALIGN_LEFT:
		dc.SelectObject(&penHlight);
		dc.MoveTo(m_rcSplit.left + 1, m_rcSplit.top);
		dc.LineTo(m_rcSplit.right - 2, m_rcSplit.top);
		dc.SelectObject(&penShadow);
		dc.MoveTo(m_rcSplit.left + 1, m_rcSplit.bottom);
		dc.LineTo(m_rcSplit.right - 2, m_rcSplit.bottom);
		break;

	case CBRS_ALIGN_RIGHT:
		dc.SelectObject(&penHlight);
		dc.MoveTo(m_rcSplit.left + 2, m_rcSplit.top);
		dc.LineTo(m_rcSplit.right - 1, m_rcSplit.top);
		dc.SelectObject(&penShadow);
		dc.MoveTo(m_rcSplit.left + 2, m_rcSplit.bottom);
		dc.LineTo(m_rcSplit.right - 1, m_rcSplit.bottom);
		break;
    }
}



void CBaseBar::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here

	//
	// Nothing to do
	//
}



/////////////////////////////////////////////////////////////////////////////
// CBaseBar operations

void CBaseBar::SetPane(BOOL bAlt, LPCTSTR lpszCaption, CWnd* pPaneWnd, BOOL bShow /*=TRUE*/)
{
    ASSERT_VALID(pPaneWnd);

	CBasePane* pPane = NULL;
	if(!bAlt)
	{
		pPane = &m_Pane1;
	}
	else
	{
		pPane = &m_Pane2;
	}

	pPane->m_pChildWnd = pPaneWnd;
	pPane->SetWindowText(lpszCaption);
	if(pPane->m_pChildWnd != NULL)
	{
		pPane->m_bShow = bShow;
	}
	else
	{
		pPane->m_bShow = FALSE;
	}

	ShowPane(bAlt, pPane->m_bShow);
}



void CBaseBar::ShowPane(BOOL bAlt, BOOL bShow)
{
	CBasePane* pPane = NULL;
	if(!bAlt)
	{
		pPane = &m_Pane1;
	}
	else
	{
		pPane = &m_Pane2;
	}

	pPane->m_bShow = bShow;
	if(pPane->m_bShow)
	{
		pPane->SendMessage(WM_SIZE);
		pPane->ShowWindow(SW_SHOW);
	}
	else
	{
		pPane->ShowWindow(SW_HIDE);
	}

	bShow = (m_Pane1.m_bShow || m_Pane2.m_bShow);
	ShowWindow(bShow ? SW_SHOW : SW_HIDE);
	SetWindowPos(NULL, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_FRAMECHANGED);
	m_pDockSite->RecalcLayout();
}



BOOL CBaseBar::PaneIsShow(BOOL bAlt)
{
	CBasePane* pPane = NULL;
	if(!bAlt)
	{
		pPane = &m_Pane1;
	}
	else
	{
		pPane = &m_Pane2;
	}

	return pPane->m_bShow;
}



/////////////////////////////////////////////////////////////////////////////
// CBaseBar internal operations

void CBaseBar::TrackSize(CPoint point)
{
	if(GetCapture() != NULL) return;


	AfxLockTempMaps();
	SetCapture();

	UINT uAlign = GetStyle() & CBRS_ALIGN_ANY;
	int nSizeInit = m_nSize;
	CPoint ptInit = point;
	CPoint ptOld = point;
	CRect rect;

	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0))
	{
		if(GetCapture() != this) break;

		switch(msg.message)
		{
		case WM_MOUSEMOVE:
			point.x = (short)LOWORD(msg.lParam);
			point.y = (short)HIWORD(msg.lParam);
			ClientToScreen(&point);
			m_pDockSite->GetClientRect(&rect);

			switch(uAlign)
			{
			case CBRS_ALIGN_TOP:
				m_nSize = nSizeInit + (point.y - ptInit.y);
				if(m_nSize < m_nMinSize)
				{
					m_nSize = m_nMinSize;
				}
				else if(m_nSize > rect.bottom - m_nMinSize)
				{
					m_nSize = rect.bottom - m_nMinSize;
				}
				break;

			case CBRS_ALIGN_BOTTOM:
				m_nSize = nSizeInit + (ptInit.y - point.y);
				if(m_nSize < m_nMinSize)
				{
					m_nSize = m_nMinSize;
				}
				break;

			case CBRS_ALIGN_LEFT:
				m_nSize = nSizeInit + (point.x - ptInit.x);
				if(m_nSize < m_nMinSize)
				{
					m_nSize = m_nMinSize;
				}
				else if(m_nSize > rect.right - m_nMinSize)
				{
					m_nSize = rect.right - m_nMinSize;
				}
				break;

			case CBRS_ALIGN_RIGHT:
				m_nSize = nSizeInit + (ptInit.x - point.x);
				if(m_nSize < m_nMinSize)
				{
					m_nSize = m_nMinSize;
				}
				break;
			}

			if(ptOld != point)
			{
				ptOld = point;
				m_pDockSite->RecalcLayout();
			}
			break;

		case WM_LBUTTONUP:
			goto ExitTrack;

		default:
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			break;
		}
	}

ExitTrack:
	ReleaseCapture();
	AfxUnlockTempMaps(FALSE);

	m_pDockSite->RecalcLayout();
}



void CBaseBar::TrackSplit(CPoint point)
{
	if(GetCapture() != NULL) return;

	AfxLockTempMaps();
	SetCapture();

	UINT uAlign = GetStyle() & CBRS_ALIGN_ANY;
	int nSplitInit = m_nSplit;
	CPoint ptInit = point;
	CPoint ptOld = point;
	CRect rect;

	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0))
	{
		if(GetCapture() != this) break;

		switch(msg.message)
		{
		case WM_MOUSEMOVE:
			point.x = (short)LOWORD(msg.lParam);
			point.y = (short)HIWORD(msg.lParam);
			ClientToScreen(&point);
			GetClientRect(&rect);

			switch(uAlign)
			{
			case CBRS_ALIGN_TOP:
			case CBRS_ALIGN_BOTTOM:
				m_nSplit = nSplitInit + (point.x - ptInit.x);
				if(m_nSplit < m_nMinSize)
				{
					m_nSplit = m_nMinSize;
				}
				else if(m_nSplit > rect.right - m_nMinSize)
				{
					m_nSplit = rect.right - m_nMinSize;
				}
				break;

			case CBRS_ALIGN_LEFT:
			case CBRS_ALIGN_RIGHT:
				m_nSplit = nSplitInit + (point.y - ptInit.y);
				if(m_nSplit < m_nMinSize)
				{
					m_nSplit = m_nMinSize;
				}
				else if(m_nSplit > rect.bottom - m_nMinSize)
				{
					m_nSplit = rect.bottom - m_nMinSize;
				}
				break;
			}

			if(ptOld != point)
			{
				ptOld = point;
				SetWindowPos(NULL, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_FRAMECHANGED);
				UpdateWindow();
			}
			break;

		case WM_LBUTTONUP:
			goto ExitTrack;

		default:
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			break;
		}
	}

ExitTrack:
	ReleaseCapture();
	AfxUnlockTempMaps(FALSE);

	SetWindowPos(NULL, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_FRAMECHANGED);
	UpdateWindow();
}



LRESULT CBaseBar::OnPaneClose(WPARAM wParam, LPARAM lParam)
{
	HWND hPaneWnd = (HWND)lParam;
	if(hPaneWnd == m_Pane1.m_hWnd)
	{
		ShowPane(FALSE, FALSE);
	}
	else if(hPaneWnd == m_Pane2.m_hWnd)
	{
		ShowPane(TRUE, FALSE);
	}

	return 0;
}



/////////////////////////////////////////////////////////////////////////////
// CBaseBar implementation

CSize CBaseBar::CalcFixedLayout(BOOL bStretch, BOOL bHorz)
{
	CSize size = CControlBar::CalcFixedLayout(bStretch, bHorz);
	if(size.cx == 0)
	{
		size.cx = m_nSize;
	}
    if(size.cy == 0)
	{
        size.cy = m_nSize;
	}

	return size;
}



CSize CBaseBar::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	return CControlBar::CalcDynamicLayout(nLength,dwMode);
}



/////////////////////////////////////////////////////////////////////////////
// CBaseBar message handlers

void CBaseBar::OnWindowPosChanged(WINDOWPOS FAR* lpwndpos) 
{
    CControlBar::OnWindowPosChanged(lpwndpos);

	if(!IsWindow(m_hWnd)) return;
	if(lpwndpos->flags & SWP_NOSIZE) return; 

	SetWindowPos(NULL, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_FRAMECHANGED);
}



void CBaseBar::OnNcCalcSize(BOOL /*bCalcValidRects*/, NCCALCSIZE_PARAMS FAR* lpncsp) 
{
	// TODO: Add your message handler code here and/or call default
	GetWindowRect(m_rcBorder);
	m_rcBorder.OffsetRect(-m_rcBorder.TopLeft());
	m_rcSplit = m_rcBorder;

	UINT uAlign = GetStyle() & CBRS_ALIGN_ANY;
	switch(uAlign)
	{
	case CBRS_ALIGN_TOP:
		lpncsp->rgrc[0].top++;
		lpncsp->rgrc[0].bottom -= BORDERSIZE;
		lpncsp->rgrc[0].left++;
		lpncsp->rgrc[0].right--;
		m_rcBorder.top = m_rcBorder.bottom - BORDERSIZE;
		m_rcSplit.bottom = m_rcBorder.top + 2;
		m_rcSplit.left += m_nSplit;
		m_rcSplit.right = m_rcSplit.left + BORDERSIZE;
		break;

	case CBRS_ALIGN_BOTTOM:
		lpncsp->rgrc[0].top += BORDERSIZE;
		lpncsp->rgrc[0].bottom--;
		lpncsp->rgrc[0].left++;
		lpncsp->rgrc[0].right--;
		m_rcBorder.bottom = m_rcBorder.top + BORDERSIZE;
		m_rcSplit.top = m_rcBorder.bottom - 2;
		m_rcSplit.left += m_nSplit;
		m_rcSplit.right = m_rcSplit.left + BORDERSIZE;
		break;

	case CBRS_ALIGN_LEFT:
		lpncsp->rgrc[0].top++;
		lpncsp->rgrc[0].bottom--;
		lpncsp->rgrc[0].left++;
		lpncsp->rgrc[0].right -= BORDERSIZE;
		m_rcBorder.left = m_rcBorder.right - BORDERSIZE;
		m_rcSplit.right = m_rcBorder.left + 2;
		m_rcSplit.top += m_nSplit;
		m_rcSplit.bottom = m_rcSplit.top + BORDERSIZE;
		break;

	case CBRS_ALIGN_RIGHT:
		lpncsp->rgrc[0].top++;
		lpncsp->rgrc[0].bottom--;
		lpncsp->rgrc[0].left += BORDERSIZE;
		lpncsp->rgrc[0].right--;
		m_rcBorder.right = m_rcBorder.left + BORDERSIZE;
		m_rcSplit.left = m_rcBorder.right - 2;
		m_rcSplit.top += m_nSplit;
		m_rcSplit.bottom = m_rcSplit.top + BORDERSIZE;
		break;

	default:
		m_rcBorder.SetRectEmpty();
		m_rcSplit.SetRectEmpty();
		break;
	}


	CRect rcPane1;
	CRect rcPane2;
	GetClientRect(&rcPane1);
	GetClientRect(&rcPane2);

	if(m_Pane1.m_bShow && m_Pane2.m_bShow)
	{
		switch(uAlign)
		{
		case CBRS_ALIGN_TOP:
		case CBRS_ALIGN_BOTTOM:
			rcPane1.right = m_nSplit - 1;
			rcPane2.left = m_nSplit + BORDERSIZE;
			break;

		case CBRS_ALIGN_LEFT:
		case CBRS_ALIGN_RIGHT:
			rcPane1.bottom = m_nSplit - 1;
			rcPane2.top = m_nSplit + BORDERSIZE;
			break;

		default:
			m_rcSplit.SetRectEmpty();
			break;
		}
	}
	else
	{
		m_rcSplit.SetRectEmpty();
	}

	if(m_Pane1.m_bShow)
	{
		m_Pane1.MoveWindow(&rcPane1);
	}
	if(m_Pane2.m_bShow)
	{
		m_Pane2.MoveWindow(&rcPane2);
	}
}



UINT CBaseBar::OnNcHitTest(CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(m_dwStyle & CBRS_SIZE_DYNAMIC)
	{
		UINT uAlign = GetStyle() & CBRS_ALIGN_ANY;

		CRect rect;
		GetWindowRect(rect);
		CPoint ptTest = point;
		ptTest.Offset(-rect.TopLeft());
		if(m_rcBorder.PtInRect(ptTest))
		{
			switch(uAlign)
			{
			case CBRS_ALIGN_TOP:		return HTBOTTOM;
			case CBRS_ALIGN_BOTTOM:		return HTTOP;
			case CBRS_ALIGN_LEFT:		return HTRIGHT;
			case CBRS_ALIGN_RIGHT:		return HTLEFT;
			}
		}
		else if(m_rcSplit.PtInRect(ptTest))
		{
			switch(uAlign)
			{
			case CBRS_ALIGN_TOP:
			case CBRS_ALIGN_BOTTOM:		return HTLEFT;
			case CBRS_ALIGN_LEFT:
			case CBRS_ALIGN_RIGHT:		return HTTOP;
			}
		}
	}

	return CControlBar::OnNcHitTest(point);
}



void CBaseBar::OnNcLButtonDown(UINT nHitTest, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	switch(nHitTest)
	{
	case HTTOP:
	case HTBOTTOM:
	case HTLEFT:
	case HTRIGHT:
		if(m_dwStyle & CBRS_SIZE_DYNAMIC)
		{
			CRect rect;
			GetWindowRect(rect);
			CPoint ptTest = point;
			ptTest.Offset(-rect.TopLeft());
			if(m_rcBorder.PtInRect(ptTest))
			{
				TrackSize(point);
			}
			else if(m_rcSplit.PtInRect(ptTest))
			{
				TrackSplit(point);
			}
		}
		return;
	}

	CControlBar::OnNcLButtonDown(nHitTest, point);
}



void CBaseBar::OnUpdateCmdUI(CFrameWnd* pTarget, int bDisableIfNoHndler)
{
	UpdateDialogControls(pTarget, bDisableIfNoHndler);
}
