// BasePane.cpp : implementation file
//
/////////////////////////////////////////////////////////////////////////////
//
//  PROJECT: Outlook Express Style
//	MODULE: Base Pane
//	VERSION: 0.1
//
//		OPERATOR: Jeong Sung-won
//		CONTACT: stupi@neoidea.com
//
//
//		RECORDS:
//			[YYYY-MM-DD] Comment
//
//		CHECK POINT:
//			() NONE
//
//		LAST UPDATE: 2002-05-21
//
//
//	Copylefts (c) 2002 Jeong Sung-won
//	All rights NOT reserved.
//

#include "stdafx.h"
#include "BasePane.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



#define CAPTIONSIZE		20

/////////////////////////////////////////////////////////////////////////////
// CBasePane

BEGIN_MESSAGE_MAP(CBasePane, CWnd)
	//{{AFX_MSG_MAP(CBasePane)
	ON_WM_NCCALCSIZE()
	ON_WM_NCPAINT()
	ON_WM_NCHITTEST()
	ON_WM_WINDOWPOSCHANGED()
	ON_WM_NCMOUSEMOVE()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// CBasePane construction/destruction

CBasePane::CBasePane()
{
	m_pChildWnd = NULL;
	m_bShow = FALSE;
}



CBasePane::~CBasePane()
{
}



BOOL CBasePane::Create(LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	CString strWndClass = AfxRegisterWndClass(CS_DBLCLKS, AfxGetApp()->LoadStandardCursor(IDC_ARROW), NULL, NULL);
	return CWnd::Create(strWndClass, lpszWindowName, dwStyle, rect, pParentWnd, NULL, NULL);
}



/////////////////////////////////////////////////////////////////////////////
// CBasePane drawings

void CBasePane::OnNcPaint() 
{
	// TODO: Add your message handler code here
	CWindowDC dc(this);

	CBrush brushBackground(GetSysColor(COLOR_3DFACE));
	CPen penHlight(PS_SOLID, 1, GetSysColor(COLOR_3DHIGHLIGHT));
	CPen penShadow(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW));

	CString strText;
	GetWindowText(strText);
	dc.FillRect(&m_rcCaption, &brushBackground);
	dc.SelectStockObject(DEFAULT_GUI_FONT);
	dc.SetBkMode(TRANSPARENT);
	CRect rcText = m_rcCaption;
	rcText.DeflateRect(5, 0);
	dc.DrawText(strText, &rcText, DT_LEFT | DT_VCENTER | DT_SINGLELINE);

	dc.SelectObject(&penHlight);
	dc.MoveTo(m_rcCaption.left, m_rcCaption.bottom);
	dc.LineTo(m_rcCaption.left, m_rcCaption.top);
	dc.LineTo(m_rcCaption.right - 1, m_rcCaption.top);
	dc.SelectObject(&penShadow);
	dc.LineTo(m_rcCaption.right - 1, m_rcCaption.bottom - 1);
	dc.LineTo(m_rcCaption.left, m_rcCaption.bottom - 1);

	CRect rcClose(0, 0, 13, 13);
	int dx = m_rcClose.Width() - rcClose.Width();
	int dy = m_rcClose.Height() - rcClose.Height();
	rcClose.OffsetRect(m_rcClose.TopLeft());
	rcClose.OffsetRect(dx / 2 + 1, dy / 2 + 1);
	CRect rcClip = rcClose;
	rcClip.DeflateRect(2, 2);
	dc.IntersectClipRect(&rcClip);
	dc.DrawFrameControl(&rcClose, DFC_CAPTION, DFCS_CAPTIONCLOSE);
	dc.SelectClipRgn(NULL);
}



/////////////////////////////////////////////////////////////////////////////
// CBasePane internal operations

BOOL CBasePane::TrackClose(CPoint point)
{
	if(GetCapture() != NULL) return FALSE;

	BOOL bClose = (SendMessage(WM_NCHITTEST, NULL, MAKELPARAM(point.x, point.y)) == HTCLOSE);
	if(!bClose) return FALSE;

	AfxLockTempMaps();
	SetCapture();

	CPoint ptOld = point;
	CWindowDC dc(this);
	dc.Draw3dRect(&m_rcClose, GetSysColor(COLOR_3DHIGHLIGHT), GetSysColor(COLOR_3DSHADOW));

	BOOL bLButtonDown = FALSE;
	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0))
	{
		if(GetCapture() != this) break;

		switch(msg.message)
		{
		case WM_LBUTTONDOWN:
			bLButtonDown = TRUE;
			dc.Draw3dRect(&m_rcClose, GetSysColor(COLOR_3DSHADOW), GetSysColor(COLOR_3DHIGHLIGHT));
			break;

		case WM_MOUSEMOVE:
		case WM_LBUTTONUP:
			point.x = (short)LOWORD(msg.lParam);
			point.y = (short)HIWORD(msg.lParam);
			ClientToScreen(&point);

			bClose = (SendMessage(WM_NCHITTEST, NULL, MAKELPARAM(point.x, point.y)) == HTCLOSE);
			if(!bClose && !bLButtonDown) goto ExitTrack;
			if(msg.message == WM_LBUTTONUP) goto ExitTrack;
			if(ptOld == point) break;

			ptOld = point;

			if(bClose && bLButtonDown)
			{
				dc.Draw3dRect(&m_rcClose, GetSysColor(COLOR_3DSHADOW), GetSysColor(COLOR_3DHIGHLIGHT));
			}
			else if(bClose && !bLButtonDown)
			{
				dc.Draw3dRect(&m_rcClose, GetSysColor(COLOR_3DHIGHLIGHT), GetSysColor(COLOR_3DSHADOW));
			}
			else
			{
				dc.Draw3dRect(&m_rcClose, GetSysColor(COLOR_3DFACE), GetSysColor(COLOR_3DFACE));
			}
			break;

		default:
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			break;
		}
	}

ExitTrack:
	ReleaseCapture();
	AfxUnlockTempMaps(FALSE);

	dc.Draw3dRect(&m_rcClose, GetSysColor(COLOR_3DFACE), GetSysColor(COLOR_3DFACE));

	return bClose && bLButtonDown;
}



/////////////////////////////////////////////////////////////////////////////
// CBasePane implementations

void CBasePane::OnNcCalcSize(BOOL /*bCalcValidRects*/, NCCALCSIZE_PARAMS FAR* lpncsp) 
{
	// TODO: Add your message handler code here and/or call default
	lpncsp->rgrc[0].top += CAPTIONSIZE;

	GetWindowRect(m_rcCaption);
	m_rcCaption.OffsetRect(-m_rcCaption.TopLeft());
	m_rcCaption.bottom = m_rcCaption.top + CAPTIONSIZE;

	m_rcClose.top = m_rcCaption.top + 2;
	m_rcClose.bottom = m_rcCaption.bottom - 2;
	m_rcClose.right = m_rcCaption.right - 2;
	m_rcClose.left = m_rcCaption.right - m_rcClose.Height() - 4;
}



/////////////////////////////////////////////////////////////////////////////
// CBasePane message handlers

void CBasePane::OnWindowPosChanged(WINDOWPOS FAR* lpwndpos) 
{
	CWnd::OnWindowPosChanged(lpwndpos);
	
	// TODO: Add your message handler code here
	if(!IsWindow(m_hWnd)) return;
	if(lpwndpos->flags & SWP_NOSIZE) return; 

	SetWindowPos(NULL, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_FRAMECHANGED);
}



void CBasePane::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	if(m_pChildWnd != NULL)
	{
		CRect rect;
		GetClientRect(&rect);
		m_pChildWnd->SetParent(this);
		m_pChildWnd->MoveWindow(&rect);
		m_pChildWnd->ShowWindow(SW_SHOW);
	}
}



UINT CBasePane::OnNcHitTest(CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CRect rect;
	GetWindowRect(rect);
	CPoint ptTest = point;
	ptTest.Offset(-rect.TopLeft());
	if(m_rcClose.PtInRect(ptTest))
	{
		return HTCLOSE;
	}
	else if(m_rcCaption.PtInRect(ptTest))
	{
		return HTCAPTION;
	}

	return CWnd::OnNcHitTest(point);
}



void CBasePane::OnNcMouseMove(UINT nHitTest, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(nHitTest == HTCLOSE)
	{
		if(TrackClose(point))
		{
			GetParent()->SendMessage(BPN_CLOSE, NULL, (LPARAM)m_hWnd);
		}
		return;
	}

	CWnd::OnNcMouseMove(nHitTest, point);
}



void CBasePane::OnNcLButtonDown(UINT nHitTest, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default

	//
	// Nothing to do
	//
}
