#include "common_header.h"
#include<iostream>
using namespace std;

#define PORT 1000

Session_Buf Sess_Buffer;

struct ThreadParameter
{
	HANDLE hPB_Mutex;
	HANDLE hTR_Event;
	HANDLE hSPS_TR_Handle;
	HANDLE hRCV_TR_Handle;
	Queue<Packet_Record>* Packet_Buffer_Pointer;
	SOCKET clntSok;
};

DWORD WINAPI RcvThread(void *arg);
DWORD WINAPI SPSThread(void *arg);

void ErrorHandling(char *message);

void main()
{
	WSADATA wsaData;
	SOCKET servSock;
	
	SOCKADDR_IN servAddr;
	SOCKADDR_IN clntAddr;
	int clntAddrSize;
	
	
	DWORD dw_RecieveThreadID;
	DWORD dw_SPSThreadID;

	//Initialize DB Module
	if(!initiateDBmodule())
	{
		cout<<"DB Connection failed!"<<endl;
		exit(0);
	}
	
	//Load Winsok 2.2 DLL
	if(WSAStartup(MAKEWORD(2, 2), &wsaData) != 0) /* Load Winsock 2.2 DLL */
		ErrorHandling("WSAStartup() error!");
	
	//Create Session Buffer Mutexes
	
	hSB_Mutex=CreateMutex(NULL, FALSE, NULL);
	if(hSB_Mutex==NULL)
		ErrorHandling("CreateMutex()SB error");


	//Create server socket
	servSock=socket(PF_INET, SOCK_STREAM, 0);   
	if(servSock == INVALID_SOCKET)
		ErrorHandling("socket() error");
	
	//server IP & Port
	memset(&servAddr, 0, sizeof(servAddr));
	servAddr.sin_family=AF_INET;
	servAddr.sin_addr.s_addr=htonl(INADDR_ANY);
	servAddr.sin_port=htons(PORT);

	//bind
	if(bind(servSock, (SOCKADDR*) &servAddr, sizeof(servAddr))==SOCKET_ERROR)
		ErrorHandling("bind() error");

	//listen
	if(listen(servSock, 5)==SOCKET_ERROR)
		ErrorHandling("listen() error");
	
	//Infinite Loop
	while(1)
	{
		ThreadParameter* TP;
		TP = new ThreadParameter;
	
		clntAddrSize=sizeof(clntAddr);
		//Accept
		TP->clntSok=accept(servSock, (SOCKADDR*)&clntAddr, &clntAddrSize);
		if(TP->clntSok==INVALID_SOCKET)
			ErrorHandling("accept() error");

		//Register Session
		Session_Record SR;
		SR.Client_Socket = TP->clntSok;
		WaitForSingleObject(hSB_Mutex, INFINITE);
			Sess_Buffer.insertSession(SR);
		ReleaseMutex(hSB_Mutex);

		//Create mutex for Packet Buffer
		TP->hPB_Mutex=CreateMutex(NULL, FALSE, NULL);
		if(TP->hPB_Mutex==NULL)
			ErrorHandling("CreateMutex()PB error");
		
		//Packet Buffer
		TP->Packet_Buffer_Pointer = new Queue<Packet_Record>;

		cout<<"connected\n";
		cout<<"Sess_Count : "<<Sess_Buffer.getCount()<<endl;

		//Create Event for thread block waiting
		TP->hTR_Event = CreateEvent(NULL, TRUE, TRUE, NULL);
	
		
		//create Sever Process & Sending Thread
		TP->hSPS_TR_Handle = CreateThread(NULL, 0, SPSThread, 
								(void*)TP, 0, &dw_SPSThreadID); 

		//create Sever Reieving Thread
		TP->hRCV_TR_Handle = CreateThread(NULL, 0, RcvThread, 
								(void*)TP, 0, &dw_RecieveThreadID);

		if(TP->hRCV_TR_Handle == 0) 
			ErrorHandling("RecieveThread error");
		if(TP->hSPS_TR_Handle == 0) 
			ErrorHandling("SPSThread error");
	}
	WSACleanup();
	return;
}// end main()

	
DWORD WINAPI RcvThread(void *arg)
{
	ThreadParameter* TP = (ThreadParameter*)arg;

	int strLen=0;
	char message[BUFSIZE];

	cout<<"Sess_Count : "<<Sess_Buffer.getCount()<<endl;
	
	while((WaitForSingleObject(TP->hSPS_TR_Handle,0) == WAIT_TIMEOUT) &&
		(strLen=recv(TP->clntSok, message, BUFSIZE, 0)) > 0 )
	{	//store packet to packet buffer

		cout<<"Packet Recieved:"<<message<<"\n";

		Packet_Record PR(strLen, message);
	    WaitForSingleObject(TP->hPB_Mutex, INFINITE);
			TP->Packet_Buffer_Pointer->Add(&PR);
		ReleaseMutex(TP->hPB_Mutex);
		// wake SPSThread
		SetEvent(TP->hTR_Event);
	}//end while

	//closing : 1. delete session 2. close socket

	cout<<"Closing:"<<TP<<endl;

	// delete Packet Buffer
	WaitForSingleObject(TP->hPB_Mutex, INFINITE);
		delete TP->Packet_Buffer_Pointer;
		TP->Packet_Buffer_Pointer = NULL;
	ReleaseMutex(TP->hPB_Mutex);
	
	cout<<"Packet Buffer Destroyed\n"<<endl;

	// wake SPSThread
	SetEvent(TP->hTR_Event);

	WaitForSingleObject(hSB_Mutex, INFINITE);
		Sess_Buffer.deleteSession(TP->clntSok);
	ReleaseMutex(hSB_Mutex);

	cout<<"Session Deleted\n"<<endl;
	
	// wait until SPSThread die
	cout<<"Wait until SPSThread die"<<endl;
	WaitForSingleObject(TP->hSPS_TR_Handle,INFINITE);

	closesocket(TP->clntSok);
	CloseHandle(TP->hPB_Mutex);
	CloseHandle(TP->hTR_Event);
	delete TP;

	cout<<"Connection Closed"<<endl;

	return 0;
}

DWORD WINAPI SPSThread(void* arg)
{
	cout<<"Sess_Count : "<<Sess_Buffer.getCount()<<endl;
	ThreadParameter* TP = (ThreadParameter*)arg;

	int queue_length;
	Packet_Record* PR;

	Packet* Pack;
	Packet* Pack2Send;

	int Processing_Result = 0;
	
	while(Processing_Result >= 0)
	{
		Processing_Result = 0;
		WaitForSingleObject(TP->hPB_Mutex, INFINITE);
			if (!TP->Packet_Buffer_Pointer)
			{//Packet Buffer Destroyed
				ReleaseMutex(TP->hPB_Mutex);
				//thread close
				break;
			}//end if
			else if (!(queue_length = TP->Packet_Buffer_Pointer->Length()))
			{//Nothing in Packet Buffer
				ReleaseMutex(TP->hPB_Mutex);

				cout<<"SPSThread BLOCKED\n";

				//Wait util packet recieved
				ResetEvent(TP->hTR_Event);
				WaitForSingleObject(TP->hTR_Event,INFINITE);
				continue;
			}//end else if
			//Retrieve a Packet from Buffer
			PR = new Packet_Record();
			TP->Packet_Buffer_Pointer->Get(PR);
		ReleaseMutex(TP->hPB_Mutex);
	
		Pack2Send = new Packet();
		
					
		cout<<"ADPing"<<endl;
		//Analyze
		if((Processing_Result = Analyze(PR->buf, 
									TP->clntSok, 
									&Pack,
									&Sess_Buffer))
			>0)
		{
		//Packet_Process
			if ((Processing_Result = Packet_Process(Pack, 
									TP->clntSok, 
									*Pack2Send,
									&Sess_Buffer))
				>0)
			{
				//Send Packet
				send(TP->clntSok, Pack2Send->GetPacketStream(),
					Pack2Send->GetPacketSize(),0);
			}// end if
		}
		else
		{
			if (Processing_Result == 0)
			{
				Pack2Send->InsertQR_Type(OSPP_ERR0);
				send(TP->clntSok, Pack2Send->GetPacketStream()
					,Pack2Send->GetPacketSize(),0);
			}//end inner if
		}//end else
		cout<<"Respond:"<<Pack2Send->GetPacketStream()<<endl;
	
		delete PR;
		delete Pack2Send;
	}// end while
	cout<<"SPSThread Destroyed"<<endl;
	return 0;
}// end SPSThread()

void ErrorHandling(char *message)
{
  fputs(message, stderr);
  fputc('\n', stderr);
  exit(1);
}

