#include "Packet_Class.h"
#include <iostream>
using namespace std;

Packet::Packet()
{
	OSP_Sig1 = OSPP_SIG1;
	OSP_Sig2 = OSPP_SIG2;
	OSP_Version = OSPP_VERS;
	Num_Field = 0;
	DLG_ID = 0;
}

Packet::Packet(const Packet& Source)
{
	OSP_Sig1 = Source.OSP_Sig1;
	OSP_Sig2 = Source.OSP_Sig2;
	OSP_Version = Source.OSP_Version;
	QR_Type = Source.QR_Type;
	DLG_ID = Source.DLG_ID;
	Num_Field = Source.Num_Field;
	for(int i = 0; i < Num_Field; i++)
		Field_Len[i] = Source.Field_Len[i];
	strcpy(message, Source.message);
}

Packet::~Packet()
{
}

Packet Packet::operator = (const Packet& Source)
{
	OSP_Sig1 = Source.OSP_Sig1;
	OSP_Sig2 = Source.OSP_Sig2;
	OSP_Version = Source.OSP_Version;
	QR_Type = Source.QR_Type;
	DLG_ID = Source.DLG_ID;
	Num_Field = Source.Num_Field;
	for(int i = 0; i < Num_Field; i++)
		Field_Len[i] = Source.Field_Len[i];
	strcpy(message, Source.message);
	return *this;
}

void Packet::InsertSignature(char* Signature)
{
	int* Sig;
	Sig = (int*) Signature;
	OSP_Sig1 = *Sig;
	Sig = (int*) (Signature + 4);
	OSP_Sig2 = *Sig;
}

void Packet::InsertSignature(unsigned int Sig1, unsigned int Sig2)
{
	OSP_Sig1 = Sig1;
	OSP_Sig2 = Sig2;
}

void Packet::InsertVersion(char* Version)
{
	int* Ver;
	Ver = (int*) Version;
	OSP_Version = *Ver;
}

void Packet::InsertVersion(unsigned int Version)
{
	OSP_Version = Version;
}

void Packet::InsertQR_Type(int QR)
{
	QR_Type = QR;
}

void Packet::InsertDLG_ID(int DID)
{
	DLG_ID = (char)DID;
}

void Packet::InsertCheckSum(int CS)
{
	Check_Sum = (char)CS;
}

void Packet::InsertHeader(char* Signature, 
						  char* Version, 
						  int QR, int DID)
{
	int* int_pointer;
	int_pointer = (int*) Signature;
	OSP_Sig1 = *int_pointer;
	int_pointer = (int*) Signature + 4;
	OSP_Sig2 = *int_pointer;
	int_pointer = (int*) Version;
	OSP_Version = *int_pointer;
	QR_Type = (char)QR;
	DLG_ID = (char)DID;
}

int Packet::AppendField(char* Field)
{
	int buf_pointer = 0;
	for (int i = 0; i < Num_Field; i++)
		buf_pointer = buf_pointer + (int)Field_Len[i];
	Field_Len[i] = (char)strlen(Field);
	for ( int j = 0; j < (int)Field_Len[i]; j++)
		message[buf_pointer+j] = Field[j];
	Num_Field ++;
	return (int)Field_Len[i];
}

int Packet::AppendField(char ch)
{
	int buf_pointer = 0;
	for (int i = 0; i < Num_Field; i++)
		buf_pointer = buf_pointer + (int)Field_Len[i];
	Field_Len[i] = (char)1;
	message[buf_pointer] = ch;
	Num_Field ++;
	return (int)Field_Len[i];
}

int Packet::AppendField(int number)
{
	int buf_pointer = 0;
	for (int i = 0; i < Num_Field; i++)
		buf_pointer = buf_pointer + (int)Field_Len[i];
	Field_Len[i] = (char)1;
	message[buf_pointer] = (char)number;
	Num_Field ++;
	return (int)Field_Len[i];
}
bool Packet::IsHeaderValid()
{
	return (OSP_Sig1 == OSPP_SIG1 && OSP_Sig2 == OSPP_SIG2
			&& OSP_Version == OSPP_VERS);
}

char* Packet::GetSignature()
{
	return (char*) OSP_Sig1;
}

char* Packet::GetVersion()
{
	return (char*) OSP_Version;
}

int Packet::GetDLG_ID()
{
	return (int)DLG_ID;
}

int Packet::GetQR_Type()
{
	return QR_Type;
}

int Packet::GetNum_Field()
{
	return (int)Num_Field;
}

int Packet::GetField(int index, char* field)
{
	int buf_pointer = 0;
	if(index > (int)Num_Field)
		return false;
	for (int i = 0; i < index; i++)
		buf_pointer = buf_pointer + (int)Field_Len[i];
	if((int)Field_Len[i] < 0 )
		return false;
	for (int j = 0; j < (int)Field_Len[i]; j++)
		field[j]  = message[buf_pointer + j];
	field[j] = (char)0;
	return true;
}

int Packet::GetField_Len(int index)
{
	return (int)Field_Len[index];
}

char* Packet::GetPacketStream()
{
	return (char*) this;
}

int Packet::GetPacketSize()
{
	int size = 0;
	size += SIGNATURE_SIZE;		//8
	size += VERSION_SIZE;		//4
	size += QR_TYPE_SIZE;		//4
	size += DLG_ID_SIZE;		//1
	size += NUM_FIELD_SIZE;		//1
	size += FIELD_LEN_MAX;		//13
	size += CHECK_SUM_SIZE;		//1

	for(int i =0; i < Num_Field; i++)
		size += Field_Len[i];
	
	return size;
}